/**
 * @fileOverview FacilityInfo contains all the "classes" for the FacilityInfoCollection, including the FacilityInfo and
 *               SecurityGroups. The FacilityInfo is lazily initialized and added to the collection on first use. See
 *               the facilityInfoTest.html for examples of how tu use the FacilityInfoCollection objects. This file
 *               depends ine inclusion of the following files in the html or aspx page where it is used.
 * 
 * <pre>
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashtable-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashset-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;rams-environment.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;dynamicGeneration.js&quot;&gt;&lt;/script&gt;
 * </pre>
 * 
 * In some cases the jshint errors are suppressed with jshint -W### codes.
 * @version 2.3.0
 */
// Constants used throughout classes
var IACUC = "IACUC";
var IRB = "IRB";
var RDC = "RDC";
var SRS = "SRS";
var SC1 = "SC1";
var SC2 = "SC2";
var SC3 = "SC3";

function FacilityInfoCollection() {
   // constructor
   // Singleton Pattern so that only one instance is created.
   if (typeof FacilityInfoCollection.instance === "object") {
      return FacilityInfoCollection.instance;
   }

   // JSHINT This file requires Hashset and Hashtable to be loaded. @see fileOverview
   // jshint -W117
   this.fic = new Hashtable();
   // proxyHolders is a 2 dimensional array indexed by proxyHolder (aka Delegated.Id) and CommitteesId. The value
   // contains a comma separated list of Facilities. The list of facilities is all the sites that delegated their
   // committee function to proxyHolder. Said another way, If I am the proxy Holder, what facility am I performing
   // committee activities for.
   // this.proxyHolders = new Hashtable();
   this.proxyHolders = {};
   FacilityInfoCollection.instance = this;
   var ficObj = this;

   // Get the list of delegated committees for use later by the proxy holder when creating their committeeInfo
   this.getProxyHolders(function() {
      // load the Central IRB by default because it may be needed.
      // JSHINT This file requires requires RAMS-environment.js to be loaded. @see fileOverview
      // JSHINT 'fi' is defined but never used. This is true in this case because there is no need to return fi to
      // whomever
      // instantiated the fic.
      // jshint -W098
      ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite, function(fi) {
         return FacilityInfoCollection.instance;
      });
   });
}
FacilityInfoCollection.prototype.getProxyHolderInfo = function(proxyHolderId, committeeType) {
   var ficObj = this;
   var proxies = [];
   var cps = {};

   if (ficObj.proxyHolders.hasOwnProperty(proxyHolderId)) {
      cps = ficObj.proxyHolders[proxyHolderId];
      if (typeof cps[committeeType] !== "undefined") {
         //console.log("getProxyHolderInfo for proxyHolderId: " + proxyHolderId + " committeeType: " + committeeType
         //   + " cps[" + committeeType + "]: " + cps[committeeType]);
         proxies = cps[committeeType];
      }
   }
   return proxies;
};

FacilityInfoCollection.prototype.getProxyHolders = function(callback) {
   var ficObj = this;
   if (typeof callback === "function") {
      // JSHINT This file requires requires RAMS-environment.js to be loaded. @see fileOverview
      // jshint -W117
      var comUrl = dynamicSharepointURL + "_api/web/lists/getbytitle('Committees')/items"
         + "?$select=Id,Title,CommitteesLookup/ID,CommitteesLookup/CommitteesName,FacilitiesLookup/ID,"
         + "FacilitiesLookup/Title,Delegated/ID,Delegated/Title,Hidden"
         + "&$expand=FacilitiesLookup,CommitteesLookup,Delegated" + "&$filter=Delegated/ID gt 0"
         + "&$orderby=Delegated/ID,Id";
      // cps = committee proxies holds the committees that a facility proxies for
      var cps = {};
      var pfa = [];
      $
         .ajax({
            url : comUrl,
            method : "GET",
            async : false,
            headers : {
               "Accept" : "application/json; odata=verbose"
            },
            success : function(data) {
               var comProxies = data.d.results;
               for (var i = 0; i < comProxies.length; i++) {
                  cps = {};
                  pfa = [];
                  if ((ficObj.proxyHolders.hasOwnProperty(comProxies[i].Delegated.ID))) {
                     // The delegated facility is in the proxy holders object so see if the committee is also present.
                     // if so, concatenate the facilities, otherwise add the committee and the facility.
                     cps = ficObj.proxyHolders[comProxies[i].Delegated.ID];
                     if ((ficObj.proxyHolders[comProxies[i].Delegated.ID]
                        .hasOwnProperty(comProxies[i].CommitteesLookup.CommitteesName))) {
                        if ((typeof cps[comProxies[i].CommitteesLookup.CommitteesName] === "undefined")
                           || (cps[comProxies[i].CommitteesLookup.CommitteesName] === "")
                           || (cps[comProxies[i].CommitteesLookup.CommitteesName] === null)) {
                           pfa.push(comProxies[i].FacilitiesLookup.ID);
                           cps[comProxies[i].CommitteesLookup.CommitteesName] = pfa;
                        } else {
                           // ... otherwise add the committee and the facility.
                           pfa = cps[comProxies[i].CommitteesLookup.CommitteesName];
                           pfa.push(comProxies[i].FacilitiesLookup.ID);
                           cps[comProxies[i].CommitteesLookup.CommitteesName] = pfa;
                        }
                     } else {
                        pfa.push(comProxies[i].FacilitiesLookup.ID);
                        cps[comProxies[i].CommitteesLookup.CommitteesName] = pfa;
                     }
                  } else {
                     // This is the first time the Delegated facilityId is added to the proxyHolders object so
                     // we know that the committee is also empty. Initialize it to the committee being processed.
                     pfa.push(comProxies[i].FacilitiesLookup.ID);
                     cps[comProxies[i].CommitteesLookup.CommitteesName] = pfa;
                  }
                  ficObj.proxyHolders[comProxies[i].Delegated.ID] = cps;
               }
               callback();
            },
            error : onError
         });
   } else {
      throw "Invalid callback function. (FacilityInfoCollection.getProxyHolders)";
   }
};
FacilityInfoCollection.prototype.get = function(facility, callback) {
   var fi;
   // need to save "this" to a variable because other functions can't rely on this being set.
   var ficObj = this;
   if (typeof callback === "function") {
      // does the facility exist in the fic?
      if (!(ficObj.fic.containsKey(facility))) {
         // JSHINT the newFacilityInfo line receives the warning "Do not use 'new' for side effects"
         // however the constructor passes the created object to the callback function. Using the JSHINT annotation
         // to remove the warning.
         // jshint -W031
         new FacilityInfo(facility, ficObj, function(returnedFI) {
            if (typeof returnedFI === "undefined") {
               throw "Facility " + facility + " is not found. (FacilityInfoCollection.get)";
            } else {
               // add the facility by name and ID so it can be found either way.
               ficObj.fic.put(returnedFI.getFacilityId(), returnedFI);
               ficObj.fic.put(returnedFI.getFacilityName(), returnedFI);
               fi = returnedFI;
               // return the requested FacilityItem
               callback(fi);
            }
         });
      } else {
         fi = ficObj.fic.get(facility);
         // return the requested FacilityItem
         callback(fi);
      }
   } else {
      throw "Invalid callback function. (FacilityInfoCollection.get)";
   }
};
FacilityInfoCollection.prototype.remove = function(facility) {
   // does the facility exist in the fic?
   if (this.fic.containsKey(facility)) {
      // remove the fic entry
      var fi = this.fic.get(facility);
      var fiId = fi.facilityId;
      // var fiName = fi.facilityName;
      this.fic.remove(fiId);
      this.fic.remove(facility);
   }
   return;
};
// ----------------------------------------------------------------------------------------------------------------
function FacilityInfo(facilityKey, ficObj, callback) {
   FacilityInfo.instance = this;

   // // Constants used throughout FacilityInfo
   // var IACUC = "IACUC";
   // var IRB = "IRB";
   // var RDC = "RDC";
   // var SRS = "SRS";
   // var SC1 = "SC1";
   // var SC2 = "SC2";
   // var SC3 = "SC3";

   this.fi = this;
   this.facilityId = "";
   this.facilityName = facilityKey;
   // this.facilityTitle = "";
   this.facilityLocation = "";
   this.facilitySiteCode = "000";
   this.facilitySubsite = "";
   this.facilityLiaison = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityLocalSiteAdministrator = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityLocalSiteLiaisons = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityAdministrativeOfficers = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityResearchComplianceOfficers = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityAnimalFacilityManagers = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };
   this.facilityUsers = {
      id : "",
      name : "",
      facilityName : "",
      committeeType : "",
      securityGroupType : ""
   };

   // associative array of committees, indexed by committeeType.
   this.committees = [];

   // need to save "this" to a variable because other functions can't rely on this being set.
   var fiObj = this;
   var onError = function onError(error) {
      alert(JSON.stringify(error));
      throw "Error retrieving Facility or Committee. (FacilityInfo.onError) " + JSON.stringify(error);
   };
   var processCommittee = function processCommittee(comData, callback) {
      var returnedComJson;
      if (typeof callback === "function") {
         if ((comData) && (comData.d) && (comData.d.results[0])) {
            returnedComJson = comData.d.results;
         } else {
            throw "No result set found for Committee " + fiObj.FacilityName + ". (FacilityInfo.processCommittee)";
         }
         var cObj;
         var cItem;
         // we should have one record for each committee at the facility.
         for (var i = 0; i < returnedComJson.length; i++) {
            cItem = returnedComJson[i];
            // var cObj = new CommitteeInfo(cItem.CommitteeType, new SecurityGroup(cItem.Administrators.ID,
            // id, ctype, name, acronym, adminGroup, coordGroup, memberGroup, delegateId, hideGroup
            try {
               cObj = new CommitteeInfo(
                  cItem.Id,
                  cItem.CommitteesLookup.CommitteesName,
                  cItem.Title,
                  cItem.Acronym,
                  new SecurityGroup(cItem.Administrators.ID, cItem.Administrators.Name),
                  new SecurityGroup(cItem.CommitteeCoordinators.ID, cItem.CommitteeCoordinators.Name),
                  new SecurityGroup(cItem.CommitteeMembers.ID, cItem.CommitteeMembers.Name),
                  (cItem.hasOwnProperty("Delegated") ? (((cItem.Delegated.hasOwnProperty("ID")) && (cItem.Delegated.ID !== -1)) ? cItem.Delegated.ID
                     : "")
                     : ""),
                  // (cItem.hasOwnProperty("delegated.Title") ? cItem.delegated.Title : "" ),
                  cItem.Hidden, ficObj, fiObj);
            } catch (err) {
               console.log(err);
               // do nothing
            }
            fiObj.committees[cItem.CommitteesLookup.CommitteesName] = cObj;
         }

         callback(fiObj);
      } else {
         throw "Invalid callback function. (FacilityInfo.processCommittee)";
      }
   };
   var processFacility = function processFacility(facData, callback) {
      var returnedFacJson;
      if (typeof callback === "function") {
         if ((facData) && (facData.d) && (facData.d.results[0])) {
            returnedFacJson = facData.d.results[0];
         } else {
            // JSHINT possible strict violation because of the use of "this"
            throw "No result set found for Facility " + this.FacilityName;
         }
         fiObj.facilityId = returnedFacJson.Id;
         fiObj.facilityName = returnedFacJson.Title;
         fiObj.facilityTitle = returnedFacJson.Title;
         fiObj.facilitySiteCode = returnedFacJson.SiteCode;
         fiObj.facilityLocation = returnedFacJson.FacilityLocation;
         fiObj.facilitySubsite = returnedFacJson.FacilitySubsite.Url;
         fiObj.facilityLiaisonId = returnedFacJson.FacilityLiaison.ID;
         fiObj.facilityLiaisonName = returnedFacJson.FacilityLiaison.Title;
         fiObj.facilityAdministrativeOfficers = new SecurityGroup(returnedFacJson.AdministrativeOfficers.ID,
            returnedFacJson.AdministrativeOfficers.Name);
         try {
            fiObj.facilityAnimalFacilityManagers = new SecurityGroup(returnedFacJson.AnimalFacilityManagers.ID,
               returnedFacJson.AnimalFacilityManagers.Name);
         } catch (err) {
            // do nothing
         }
         fiObj.facilityLocalSiteAdministrator = new SecurityGroup(returnedFacJson.LocalSiteAdministrators.ID,
            returnedFacJson.LocalSiteAdministrators.Name);
         try {
            fiObj.facilityLocalSiteLiaisons = new SecurityGroup(returnedFacJson.LocalSiteLiaisons.ID,
               returnedFacJson.LocalSiteLiaisons.Name);
         } catch (err) {
            // do nothing
         }
         fiObj.facilityResearchComplianceOfficers = new SecurityGroup(returnedFacJson.ResearchComplianceOfficers.ID,
            returnedFacJson.ResearchComplianceOfficers.Name);
         fiObj.facilityUsers = new SecurityGroup(returnedFacJson.Users.ID, returnedFacJson.Users.Name);

         // callback(fiObj); // remove me when committee is working.
         // lookup CommitteeList info based on facilityKey and link to CommitteeList and populate attributes
         // https://vaww.idev.ram.msp.va.gov/sites/IRBApplications/_api/Web/Lists/GetByTitle('Committee%20List')/Items?$filter=Title
         // eq 'Miami' or Id eq 'Miami'
         // or
         // https://vaww.idev.ram.msp.va.gov/sites/IRBApplications/_api/Web/Lists/GetByTitle('Committee%20List')/Items?$filter=Id
         // eq CommitteeLookupId

         // &$select=Id,Title,FacilityLookup/ID,
         // FacilityLookup/FacilityName,Administrators/ID,Administrators/Name,CommitteeCoordinators/ID,CommitteeCoordinators/Name,
         // CommitteeMembers/ID,CommitteeMembers/Name,DelegatedTo/ID,DelegatedTo/Name,Hidden
         // &$expand=FacilityLookup,Administrators,CommitteeCoordinators,CommitteeMembers,DelegatedTo

         // JSHINT This file requires requires RAMS-environment.js to be loaded. @see fileOverview
         // jshint -W117
         var comUrl = dynamicSharepointURL + "_api/web/lists/getbytitle('Committees')/items" + "?$filter="
         // + "FacilityLookup/ID eq '" + fiObj.facilityId + "' "
         // + "or "
         + "FacilitiesLookup/Title eq '" + fiObj.facilityName + "' " + "&$select=Id,Title,"
            + "CommitteesLookup/ID,CommitteesLookup/CommitteesName,Acronym,"
            + "FacilitiesLookup/ID, FacilitiesLookup/Title," + "Administrators/ID,Administrators/Name,"
            + "CommitteeCoordinators/ID,CommitteeCoordinators/Name," + "CommitteeMembers/ID,CommitteeMembers/Name,"
            + "Delegated/ID,Delegated/Title," + "Hidden" + "&$expand=FacilitiesLookup,CommitteesLookup,"
            + "Administrators,CommitteeCoordinators,CommitteeMembers," + "Delegated";
         $.ajax({
            url : comUrl,
            method : "GET",
            async : false,
            headers : {
               "Accept" : "application/json; odata=verbose"
            },
            success : function(data) {
               processCommittee(data, callback);
            },
            error : onError
         });
      } else {
         throw "Invalid callback function. (FacilityInfoCollection.processFacility)";
      }
   };

   if (typeof callback === "function") {
      // constructor
      // lookup facilityList info based on facilityKey and populate attributes
      // https://vaww.idev.ram.msp.va.gov/sites/IRBApplications/_api/web/lists/getbytitle('Facility')/items?
      // $filter=FacilityName%20eq%20%27Central%20IRB%27%20or%20Id%20eq%20%27Central%20IRB%27&
      // $select=Id,FacilityName,Title,FacilityLocation,FacilityLiaison/ID,FacilityLiaison/Name,FacilityLiaison/Title,
      // FacilitySubsite,AdministrativeOfficers/ID,AdministrativeOfficers/Name,LocalSiteAdministrators/ID,LocalSiteAdministrators/Name,
      // LocalSiteLiaisons/ID,LocalSiteLiaisons/Name,ResearchComplianceOfficers/ID,ResearchComplianceOfficers/Name,Users/ID,Users/Name&
      // $expand=FacilityLiaison,AdministrativeOfficers,LocalSiteAdministrators,LocalSiteLiaisons,ResearchComplianceOfficers,Users
      // JSHINT This file requires requires RAMS-environment.js to be loaded. @see fileOverview
      // jshint -W117
      var facUrl = dynamicSharepointURL
         + "_api/web/lists/getbytitle('Facilities')/items"
         + "?$filter=Title eq '"
         + facilityKey
         + "' or Id eq '"
         + facilityKey
         + "' "
         + "&$select=Id,Title,FacilityLocation,FacilityLiaison/ID,FacilityLiaison/Name,FacilityLiaison/Title,"
         + "FacilitySubsite,AdministrativeOfficers/ID,AdministrativeOfficers/Name,"
         + "AnimalFacilityManagers/ID,AnimalFacilityManagers/Name,"
         + "LocalSiteAdministrators/ID,LocalSiteAdministrators/Name,"
         + "LocalSiteLiaisons/ID,LocalSiteLiaisons/Name,ResearchComplianceOfficers/ID,ResearchComplianceOfficers/Name,Users/ID,Users/Name"
         + "&$expand=FacilityLiaison,AdministrativeOfficers," + "AnimalFacilityManagers,"
         + "LocalSiteAdministrators,LocalSiteLiaisons,ResearchComplianceOfficers,Users";
      $.ajax({
         url : facUrl,
         method : "GET",
         async : false,
         headers : {
            "Accept" : "application/json; odata=verbose"
         },
         success : function(data) {
            // fiObj.facilityId = 123;
            processFacility(data, callback);
         },
         error : onError
      });
   } else {
      throw "Invalid callback function. (FacilityInfo.constructor)";
   }
}
FacilityInfo.prototype.getFacilityId = function() {
   return this.facilityId;
};
FacilityInfo.prototype.getFacilityName = function() {
   return this.facilityName;
};
FacilityInfo.prototype.getFacilityTitle = function() {
   return this.facilityTitle;
};
FacilityInfo.prototype.getFacilitySiteCode = function() {
   return this.facilitySiteCode;
};
FacilityInfo.prototype.getFacilityLocation = function() {
   return this.facilityLocation;
};
FacilityInfo.prototype.getFacilitySubsiteUrl = function() {
   return this.facilitySubsite;
};
// FacilityInfo.prototype.getFacilityCommitteeLookupId = function() {
// return this.facilityCommitteeLookupId;
// }
FacilityInfo.prototype.getFacilityGroup = function(group) {
   var lcGroup = group.toLowerCase();
   switch (lcGroup) {
   case "administrativeofficers":
      return this.facilityAdministrativeOfficers;
      // break;
   case "liaison":
      return this.facilityLiaison;
      // break;
   case "localsiteadministrator":
      return this.facilityLocalSiteAdministrator;
      // break;
   case "localsiteliaisons":
      return this.facilityLocalSiteLiaisons;
      // break;
   case "researchcomplianceofficers":
      return this.facilityResearchComplianceOfficers;
      // break;
   case "users":
      return this.facilityUsers;
      // break;
   }
   return {};
};
FacilityInfo.prototype.getCommitteeGroup = function(committee) {
   var ucCommittee = committee.toUpperCase();
   switch (ucCommittee) {
   case IACUC:
      return this.committees[IACUC];
      // break;
   case IRB:
      return this.committees[IRB];
      // break;
   case RDC:
      return this.committees[RDC];
      // break;
   case SRS:
      return this.committees[SRS];
      // break;
   case SC1:
      return this.committees[SC1];
      // break;
   case SC2:
      return this.committees[SC2];
      // break;
   case SC3:
      return this.committees[SC3];
      // break;
   default:
      throw "Unknown committee (" + committee + ")";
      // break;
   }
};

// ----------------------------------------------------------------------------------------------------------------
/**
 * The Committee Info object holds the attributes of a given facility's committee in the Committee List. This method is
 * the constructor where all attribute are required for instantiation.
 * 
 * @param id {number} the committee id from the committee list.
 * @param ctype {string} the type of committee as specified in the committeeType list.
 * @param name {string} the long name of the committee for display on pages and documents.
 * @param adminGroup {object} object that contains the administrator security group object, containing the security
 *           group id and name.
 * @param coordGroup {object} object that contains the coordinator security group object, containing the security group
 *           id and name.
 * @param memberGroup {object} object that contains the member security group object, containing the security group id
 *           and name.
 * @param delegateId {number} the id of the facility where the facility delegated its committee function to.
 * @param hideGroup {Boolean} indicates if the committee function is hidden at the given facility.
 */
function CommitteeInfo(id, ctype, name, acronym, adminGroup, coordGroup, memberGroup, delegateId, hideGroup, ficObj,
   fiObj) {
   // // Constants used throughout FacilityInfo
   // var IACUC = "IACUC";
   // var IRB = "IRB";
   // var RDC = "RDC";
   // var SRS = "SRS";
   // var SC1 = "SC1";
   // var SC2 = "SC2";
   // var SC3 = "SC3";
   var ucType = ctype.toUpperCase();
   if ((ucType === IACUC || ucType === IRB || ucType === RDC || ucType === SRS || ucType === SC1 || ucType === SC2 || ucType === SC3)
      && (adminGroup instanceof SecurityGroup)
      && (coordGroup instanceof SecurityGroup)
      && (memberGroup instanceof SecurityGroup)) {
      if (typeof id === "number") {
         this.committeeId = id;
      } else {
         throw "Unable to create committee because id (" + id + ") is not a number.";
      }
      this.committeeType = ucType;
      this.committeeName = name;
      this.committeeAcronym = acronym;
      this.administrators = adminGroup;
      this.coordinators = coordGroup;
      this.members = memberGroup;
      if (typeof delegateId === "number") {
         this.delagateId = delegateId;
      } else {
         this.delagateId = "";
      }
      if ((typeof hideGroup === "undefined") || (hideGroup !== true)) {
         this.committeeHidden = false;
      } else {
         this.committeeHidden = hideGroup;
      }
      this.proxyForFacilities = ficObj.getProxyHolderInfo(fiObj.getFacilityId(), ucType);
   } else {
      throw "Unable to create committee " + ucType;
   }
   return;
}
/**
 * Returns the ID of the Committee record in the committee list.
 * 
 * @returns Id {number} the id
 */
CommitteeInfo.prototype.getCommitteeId = function() {
   return this.committeeId;
};
/**
 * Returns the committeeType text for the committee as defined in the CommitteeType list.
 * 
 * @returns committeeType {string} the committeeType
 */
CommitteeInfo.prototype.getCommitteeType = function() {
   return this.committeeType;
};
/**
 * Returns the committeeName text for the committee. This is the long name and will contain (SC1, SC2, or SC3) for the
 * supplemental committees.
 * 
 * @returns committeeName {string} the acronym for the committee
 */
CommitteeInfo.prototype.getCommitteeName = function() {
   return this.committeeName;
};
/**
 * Returns the committeeAcronym text for the committee and will contain (SC1, SC2, or SC3) for the supplemental
 * committees.
 * 
 * @returns committeeAcronym {string} the acronym for the committee
 */
CommitteeInfo.prototype.getCommitteeAcronym = function() {
   return this.committeeAcronym;
};
/**
 * Returns the committee administrators object that contains id and name.
 * 
 * @returns members {object} the committee administrators object
 */
CommitteeInfo.prototype.getAdministrators = function() {
   return this.administrators;
};
/**
 * Returns the committee coordinators object that contains id and name.
 * 
 * @returns members {object} the committee coordinators object
 */
CommitteeInfo.prototype.getCoordinators = function() {
   return this.coordinators;
};
/**
 * Returns the committee members object that contains id and name.
 * 
 * @returns members {object} the committee members object
 */
CommitteeInfo.prototype.getMembers = function() {
   return this.members;
};
/**
 * Returns the facilityID that is committe function is delegated to.
 * 
 * @returns {number} the id of the facility delegated to.
 */
CommitteeInfo.prototype.getDelegateId = function() {
   return this.delagateId;
};
/**
 * Evaluates if the Committee is delegated for the facility.
 * 
 * @returns {Boolean}
 */
CommitteeInfo.prototype.isCommitteeDelegated = function() {
   if (this.delagateId) {
      return true;
   } else {
      return false;
   }
};
/**
 * Evaluates if the Committee is hidden for the facility.
 * 
 * @returns {Boolean}
 */
CommitteeInfo.prototype.isCommitteeHidden = function() {
   if (this.committeeHidden) {
      return true;
   } else {
      return false;
   }
};
/**
 * returns the proxyForFacilities array.
 * 
 * @returns {Array} proxyForFacilities
 */
CommitteeInfo.prototype.getProxyFor = function() {
   return this.proxyForFacilities;
};
// ----------------------------------------------------------------------------------------------------------------
/**
 * The Security Group object contains the id and name of the security group. in addition is parses the Security Group
 * name to determine the Facility, committeeType and Security Group.
 * 
 * @param id {number} the spUserId of the Security Group
 * @param name {string} the name of the security group.
 */
function SecurityGroup(id, name) {
   // Constants for securityGroupType
   var SGTYPE_AO = "Administrative Officers";
   var SGTYPE_AFM = "Animal Facility Managers";
   var SGTYPE_LSA = "Local Site Administrators";
   var SGTYPE_LSL = "Local Site Liaisons";
   var SGTYPE_RCO = "Research Compliance Officers";
   var SGTYPE_USERS = "Users";
   var SGTYPE_ADMIN = "Administrators";
   var SGTYPE_COORDINATORS = "Committee Coordinators";
   var SGTYPE_MEMBERS = "Committee Members";
   // // Constants used for CommitteeType
   // var IACUC = "IACUC";
   // var IRB = "IRB";
   // var RDC = "RDC";
   // var SRS = "SRS";
   // var SC1 = "SC1";
   // var SC2 = "SC2";
   // var SC3 = "SC3";

   if ((typeof id === "undefined") && (typeof name === "undefined")) {
      throw "Unable to create security group because id (" + id + ") and name (" + name + ") are undefined.";
   } else if (typeof id === "number") {
      this.id = id;
   } else {
      throw "Unable to create security group because id (" + id + ") is not a number.";
   }

   // the id will be -1 if the committee is hidden for the facility
   if (id < 0) {
      this.name = undefined;
   } else {
      this.name = name;
   }
   // sgPos is the position of the beginning of the security group type in the name field
   var sgPos = 0;
   // sgNoType is the security group name with the security group type stripped off the end.
   var sgNoType = "";

   this.facilityName = undefined;

   // this logic evaluates the security group name to determine the components. Security group names are composed of
   // either <FacilityName><CommitteeType><SecurityGroupType> or <facilityName><SecurityGroupType>, with a space
   // separating the components, e.g. Miami IRB Administrators, Asheville Users, or Central IRB IRB Members.
   if (typeof this.name !== "undefined") {
      // The first group for if statements is looking for the facility level security groups (SGTYPE_AO - SGTYPE_USERS).
      // The 3 if statements for SGTYPE_ADMIN, SGTYPE_COORDINATORS, and SGTYPE_MEMBERS are the committee level security
      // groups.
      if (name.lastIndexOf(SGTYPE_AO) !== -1) {
         this.securityGroupType = SGTYPE_AO;
         sgPos = name.lastIndexOf(SGTYPE_AO);
         sgNoType = name.replace(SGTYPE_AO, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_AFM) !== -1) {
         this.securityGroupType = SGTYPE_AFM;
         sgPos = name.lastIndexOf(SGTYPE_AFM);
         sgNoType = name.replace(SGTYPE_AFM, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_LSA) !== -1) {
         this.securityGroupType = SGTYPE_LSA;
         sgPos = name.lastIndexOf(SGTYPE_LSA);
         sgNoType = name.replace(SGTYPE_LSA, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_LSL) !== -1) {
         this.securityGroupType = SGTYPE_LSL;
         sgPos = name.lastIndexOf(SGTYPE_LSL);
         sgNoType = name.replace(SGTYPE_LSL, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_RCO) !== -1) {
         this.securityGroupType = SGTYPE_RCO;
         sgPos = name.lastIndexOf(SGTYPE_RCO);
         sgNoType = name.replace(SGTYPE_RCO, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_USERS) !== -1) {
         this.securityGroupType = SGTYPE_USERS;
         sgPos = name.lastIndexOf(SGTYPE_USERS);
         sgNoType = name.replace(SGTYPE_USERS, "").trim();
         this.facilityName = sgNoType;
      } else if (name.lastIndexOf(SGTYPE_ADMIN) !== -1) {
         this.securityGroupType = SGTYPE_ADMIN;
         sgPos = name.lastIndexOf(SGTYPE_ADMIN);
         sgNoType = name.replace(SGTYPE_ADMIN, "").trim();
      } else if (name.lastIndexOf(SGTYPE_COORDINATORS) !== -1) {
         this.securityGroupType = SGTYPE_COORDINATORS;
         sgPos = name.lastIndexOf(SGTYPE_COORDINATORS);
         sgNoType = name.replace(SGTYPE_COORDINATORS, "").trim();
      } else if (name.lastIndexOf(SGTYPE_MEMBERS) !== -1) {
         this.securityGroupType = SGTYPE_MEMBERS;
         sgPos = name.lastIndexOf(SGTYPE_MEMBERS);
         sgNoType = name.replace(SGTYPE_MEMBERS, "").trim();
      } else if (name.length <= sgNoType.length) {
         // if the length of the sgNoType is not less than the name, it means we didn't find the sgType and strip it
         // off.
         throw "Unable to set security group name (" + name + ") because the security group is not known.";
      }
   } else {
      // The name is null or undefined so set the securityGroupType to undefined.
      this.securityGroupType = undefined;
   }

   // If the facilityName is still undefined, we found a committee level security group and need to see which one and
   // finish setting the facilityName.
   if ((typeof this.facilityName === "undefined") && (typeof this.securityGroupType !== "undefined")) {
      // sgNoCom is the security group name with the committee stripped off the end.
      var sgNoCom = "";
      // lioCommitteeType is the lastindexof the committee type
      var lioCommitteeType = 0;
      if (sgNoType.lastIndexOf(IACUC) !== -1) {
         this.committeeType = IACUC;
         lioCommitteeType = sgNoType.lastIndexOf(IACUC);
         sgPos = sgNoType.lastIndexOf(IACUC);
         sgNoCom = sgNoType.replace(IACUC, "").trim();
      } else if (sgNoType.lastIndexOf(RDC) !== -1) {
         this.committeeType = RDC;
         lioCommitteeType = sgNoType.lastIndexOf(RDC);
         sgPos = sgNoType.lastIndexOf(RDC);
         sgNoCom = sgNoType.replace(RDC, "").trim();
      } else if (sgNoType.lastIndexOf(SRS) !== -1) {
         this.committeeType = SRS;
         lioCommitteeType = sgNoType.lastIndexOf(SRS);
         sgPos = sgNoType.lastIndexOf(SRS);
         sgNoCom = sgNoType.replace(SRS, "").trim();
      } else if (sgNoType.lastIndexOf(SC1) !== -1) {
         this.committeeType = SC1;
         lioCommitteeType = sgNoType.lastIndexOf(SC1);
         sgPos = sgNoType.lastIndexOf(SC1);
         sgNoCom = sgNoType.replace(SC1, "").trim();
      } else if (sgNoType.lastIndexOf(SC2) !== -1) {
         this.committeeType = SC2;
         lioCommitteeType = sgNoType.lastIndexOf(SC2);
         sgPos = sgNoType.lastIndexOf(SC2);
         sgNoCom = sgNoType.replace(SC2, "").trim();
      } else if (sgNoType.lastIndexOf(SC3) !== -1) {
         this.committeeType = SC3;
         lioCommitteeType = sgNoType.lastIndexOf(SC3);
         sgPos = sgNoType.lastIndexOf(SC3);
         sgNoCom = sgNoType.replace(SC3, "").trim();
      } else if (sgNoType.lastIndexOf(IRB) !== -1) {
         this.committeeType = IRB;
         lioCommitteeType = sgNoType.lastIndexOf(IRB);
         sgPos = sgNoType.lastIndexOf(IRB);
         sgNoCom = sgNoType.slice(0, sgPos).trim();
      } else if (sgNoType.length <= sgNoCom.length) {
         // if the length of the sgNoType is not less than the name, it means we didn't find the sgType and strip it
         // off.
         throw "Unable to set security group committeeType (" + name + ") because the type is not known.";
      }
      this.facilityName = sgNoCom;
   } else {
      this.committeeType = "";
   }

}
/**
 * Returns the id associated to the security group. This is the SPUserID of the security Group
 * 
 * @returns id {number} the spUserId of the securityGroup.
 */
SecurityGroup.prototype.getId = function() {
   return this.id;
};
/**
 * Returns the full name of the security Group
 * 
 * @returns name {string} the full name of the security group
 */
SecurityGroup.prototype.getName = function() {
   return this.name;
};
/**
 * Returns the Facility Name extracted from the Security Group Name
 * 
 * @returns facilityName {string} the name of the facility
 */
SecurityGroup.prototype.getFacilityName = function() {
   return this.facilityName;
};
/**
 * Returns the CommitteeType from the Security Group Name. this can be any of the following:
 * <ul>
 * <li>IACUC
 * <li>IRB
 * <li>RDC
 * <li>SRS
 * <li>SC1
 * <li>SC2
 * <li>SC3
 * </ul>
 * 
 * @returns committeeType {string} the type of committee
 */
SecurityGroup.prototype.getCommitteeType = function() {
   return this.committeeType;
};
/**
 * Return the object's security Group Type. this can be any of the following:
 * <ul>
 * <li>Administrative Officers
 * <li>Animal Facility Managers
 * <li>Local Site Administrators
 * <li>Local Site Liaisons
 * <li>Research Compliance Officers
 * <li>Users
 * <li>Administrators
 * <li>Committee Coordinators
 * <li>Committee Members
 * </ul>
 * 
 * @returns securityGroupType {string} the type of security group
 */
SecurityGroup.prototype.getSecurityGroupType = function() {
   return this.securityGroupType;
};
// ----------------------------------------------------------------------------------------------------------------
String.prototype.trim = function() {
   return this.replace(/^\s+|\s+$/g, "");
};
/*
 * //Usage: var fic = new FacilityInfoCollection(); var fiMiamiByName = fic.get("Miami"); var fiMiamiByNumber =
 * fic.get(5); var fiMiamiIRBInfo = fiMiamiByName.getCommtitteeGroups("IRB"); var fiMiamiIRBAdministratorsId =
 * fiMiamiIRBInfo.getAdministrators().getName(); var fiMiamiIACUCisCommitteeDelegated =
 * fiMiamiByName.getCommtitteeGroups("IACUC").isCommitteeDelegated();
 */

