#!/bin/bash
function myHelp () {
# Using a here doc with standard out.
cat <<-END

Help for $0

Deploys the RAMS application from the deploy folder in \$CATALINA_HOME to the correct tomcat locations. It accepts 
the name of the application as a parameters and this must match the name of the war file and the prefix of the xml 
configudation file. The script validates that it can find the files to deploy and their destination directories, 
stops tomcat if it is running, copies the files, and verifies that they are in their destination directories.

Usage:
------
   $0 -h | --help
     Display this help
   $0 <RAMS-2.x.0>
     Run $0 to deploy the RAMS-2.x.0 application and configuration. Replace the 2.x.0 with the actual name of the 
     war file without the extension, e.g. RAMS-2.2.0
     
Example: 
--------
     $0 RAMS-2.2.0

END
}

if [ $# -eq 1 ] 
then
    if [[ "$1" == "-h" ]] || [[ "$1" == "--help" ]] || [[ "$1" == "-help" ]]
	then
		myHelp
		exit 0
	elif [ -z "$1" ]
		then 
			BASENAME=RAMS-2.3.0
			echo "INFO: Beginning deployment using default application name $BASENAME"
		else
			BASENAME=$1
			echo "INFO: Beginning deployment of $BASENAME"
	fi
else 
	echo "ERROR: Received $# parameters when expecting the release to deploy."
	myHelp
	exit 1
fi

TODAY=$(date +%Y%m%d%H%M%S)
echo "INFO: TODAY=$TODAY"

#if [ -z "$1" ]
#then 
#	BASENAME=RAMS-2.2.0
#	echo "INFO: Beginning deployment using default application name $BASENAME"
#else
#	BASENAME=$1
#	echo "INFO: Beginning deployment of $BASENAME"
#fi



#See if Tomcat home directory, $CATALINA_HOME, is defined and if it exists
if [ -z ${CATALINA_HOME+x} ] 
then
   echo "INFO: \$CATALINA_HOME is not set - defaulting to /usr/share/apache-tomcat-7.0.57" 
   CATALINA_HOME=/usr/share/apache-tomcat-7.0.57
else 
   echo "INFO: using \$CATALINA_HOME environment variable value $CATALINA_HOME"
fi

if [ -d $CATALINA_HOME ]
then
   echo "INFO: \$CATALINA_HOME found"
else 
   echo "INFO: \$CATALINA_HOME not found at $CATALINA_HOME"
fi

if [ ! -d $CATALINA_HOME/backup ]
then 
   echo "INFO: creating \$CATALINA_HOME/backup" 
   mkdir -p $CATALINA_HOME/backup
else 
   echo "INFO: \$CATALINA_HOME/backup found"
fi

if [ ! -d $CATALINA_HOME/deploy ]
then 
   echo "INFO: creating \$CATALINA_HOME/deploy" 
   mkdir -p $CATALINA_HOME/deploy
else 
   echo "INFO: \$CATALINA_HOME/deploy found"
fi

((ERR=0))
if [ -f "$CATALINA_HOME/deploy/$BASENAME.xml" ]
then 
   echo "INFO: $BASENAME.xml file found in \$CATALINA_HOME/deploy" 
else 
   echo "ERROR: no $BASENAME.xml file found in \$CATALINA_HOME/deploy"
   ((ERR++))
fi

if [ -f "$CATALINA_HOME/deploy/$BASENAME.war" ]
then 
   echo "INFO: $BASENAME.war file found in \$CATALINA_HOME/deploy" 
else 
   echo "ERROR: no $BASENAME.war file found in \$CATALINA_HOME/deploy"
   ((ERR++))
fi

if [ $ERR = 0 ]
then
   # The output from service tomcat7 status will be something like 
   # "Tomcat servlet engine is running with pid 14416." Look in the 8th word for the pid.
   pid=$(service tomcat7 status | cut -d ' ' -f 8)
   # If the pid is not empty stop tomcat
   if [ ! -z "$pid" ]; then
      echo "INFO: stopping tomcat (pid=$pid) ..."
      service tomcat7 stop
   else
      echo "INFO: tomcat is already stopped."
   fi
   #pid=$(ps x | grep "$CATALINA_HOME" | grep -v grep | cut -d ' ' -f 1)
   #if [ ! -z "$pid" ]; then
	#  echo "INFO: stopping tomcat (pid=$pid) ..."
    #  service tomcat7 stop
   #fi

   echo "INFO: deploying the $BASENAME.xml and $BASENAME.war files"
   cp $CATALINA_HOME/deploy/$BASENAME.xml $CATALINA_HOME/conf/Catalina/localhost
   cp $CATALINA_HOME/deploy/logback.xml $CATALINA_HOME/lib
   cp $CATALINA_HOME/deploy/$BASENAME.war $CATALINA_HOME/webapps
   chown apache:apache $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml
   chmod 644 $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml
   chown apache:apache $CATALINA_HOME/lib/logback.xml
   chmod 664 $CATALINA_HOME/lib/logback.xml
   chown apache:apache $CATALINA_HOME/webapps/$BASENAME.war
   chmod 774 $CATALINA_HOME/webapps/$BASENAME.war
 
   CODEC=commons-codec-1.9.jar
   echo "INFO: deploying the $CODEC files"
   cp $CATALINA_HOME/deploy/$CODEC $CATALINA_HOME/lib/$CODEC
   cp $CATALINA_HOME/deploy/logback-core-*.jar $CATALINA_HOME/lib
   cp $CATALINA_HOME/deploy/logback-classic-*.jar $CATALINA_HOME/lib
   chmod 644 $CATALINA_HOME/lib/$CODEC
   chmod 644 $CATALINA_HOME/lib/logback-*
   chown apache:apache $CATALINA_HOME/lib/$CODEC 
   chown apache:apache $CATALINA_HOME/lib/logback*
   echo "INFO: verifying deployment"
else
   echo "ERROR: error found - nothing deployed"
   exit $ERR
fi   

rc=0
echo "INFO: verifying that the files are in the backup location and no longer in the deployed location"
if [ ! -f $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml ]
then 
   echo "ERROR: $BASENAME.xml does not exist in \$CATALINA_HOME/conf/Catalina/localhost directory"
   rc=1
fi
if [ ! -f $CATALINA_HOME/webapps/$BASENAME.war ]
then 
   echo "ERROR: $BASENAME.war does not exist in \$CATALINA_HOME/webapps directory"
   rc=2
fi
if [ ! -f $CATALINA_HOME/lib/logback.xml ]
then 
   echo "ERROR: logback.xml does not exist in \$CATALINA_HOME/lib directory"
   rc=3
fi
if [ ! -f $CATALINA_HOME/lib/$CODEC ]
then 
   echo "ERROR: $CODEC not found in \$CATALINA_HOME/lib directory"
   rc=4
fi
if [ ! -f $CATALINA_HOME/lib/logback-core-*.jar ]
then 
   echo "ERROR: logback-core-*.jar not found in \$CATALINA_HOME/lib directory"
   rc=5
fi
if [ ! -f $CATALINA_HOME/lib/logback-classic-*.jar ]
then 
   echo "ERROR: logback-classic-*.jar not found in \$CATALINA_HOME/lib directory"
   rc=6
fi

echo "INFO: Deployment Done - exit code=$rc"
CATALINAOUT=$CATALINA_HOME/logs/catalina.out
if [[ $rc -eq 0 ]]
then
   while true
   do
     # (1) prompt user, and read command line argument
     read -t 10 -n 1 -p "Do you want to delete, rename, or leave the logs/catalina.out in place [D | R | L]? " ANSWER
   
     # (2) handle the input we were given
     case $ANSWER in
      [dD]* ) rm $CATALINAOUT
              echo
			  echo "Deleted $CATALINAOUT."
              break;;
   
      [lL]* ) echo
              echo "Leaving $CATALINAOUT in place."
              break;;
   
      * )     mv $CATALINAOUT $CATALINAOUT-$TODAY
              echo
              echo "Moved $CATALINAOUT to $CATALINAOUT-$TODAY."
              break;;
     esac
   done

	echo
	echo "INFO: The next step is to run: "
	echo "      sudo -E service tomcat7 start"
    echo "      tail -f logs/catalina.out"
	echo
fi
exit $rc
