if ((Get-PSSnapin "Microsoft.SharePoint.PowerShell" -ErrorAction SilentlyContinue) -eq $null) {
    Add-PSSnapin "Microsoft.SharePoint.PowerShell"
}

$Global:PSVersion = $PSVersionTable
$ConfirmPreference = "None"

function RemoveNonNzero-RAMS {
    $siteUsers = (Get-SiteWeb).SiteUsers 
    $usersTobeRemoved = New-Object System.Collections.ArrayList

    foreach($user in $siteUsers) 
    {        
        if($user.IsSiteAdmin -eq $true) 
        { 
            if(($user.Displayname -imatch "0") -or ($user.Displayname -imatch "super") -or ($user.Displayname -imatch "service") -or ($user.Displayname -imatch "account")) {
                Write-Host "Kept: Admin: ", $user.Displayname -ForegroundColor Green
            }           
            else {
                $usersTobeRemoved.Add($user.ID)             
                Write-Host "Removed: Non-Admin:", $user.DisplayName -ForegroundColor Yellow
            }
        } 
    }    
    foreach ($id in $usersTobeRemoved) {
        $siteUsers.RemoveByID($id)
    }
}

function Remove-AOGroup {
$groups = (Get-SiteWeb).SiteGroups
$aoGroup = $groups | Where-Object {$_.Name -match "AO Group"}
Write-Host $aoGroup.Name Users: -ForegroundColor Cyan
$aoGroup.Users | Select-Object -Property UserLogin, DisplayName | Format-Table -Wrap 

if($aoGroup -eq $null) {
    Write-Host No AO Group Found -ForegroundColor Yellow
}
else {
    $groups.RemoveByID($aoGroup.ID)
    Write-Host $aoGroup.Name has been removed -ForegroundColor Yellow
    }
}

function Install-RAMS {
    #Farm account
    $Global:farmUserID = $Global:hostInfo.farmUser.GetValue(0)
    $Global:farmPwd = $Global:hostInfo.farmUser.GetValue(1)
    $Global:cred = $Null
    #ask if the serve info is missing.
    if ($farmUserID -eq "") {
        $Global:cred = Get-Credential -Message "Please Enter Valid UserID/PwD for Farm Account" 
        $Global:farmUserID = $cred.UserName 
        $Global:farmPwd = ($Global:cred.GetNetworkCredential()).Password
    }
    #Super Reader account
    $Global:superReaderUserID = $Global:hostInfo.superReader.GetValue(0)
    $Global:superReaderPwd = $Global:hostInfo.superReader.GetValue(1)
    #ask if the serve info is missing.
    if ($superReaderUserID -eq "") {
        $Global:cred = Get-Credential -Message "Please Enter Valid UserID/PwD for superReader Account" 
        $Global:superReaderUserID = $cred.UserName
        $Global:superReaderPwd = ($Global:cred.GetNetworkCredential()).Password
    }

    $Global:reportTargetServerName = $Global:hostInfo.ssrs #Target SSRS Server
    $Global:sqlServerName = $Global:hostInfo.sql #SQL Server
 
    $Global:siteUrl = Get-SiteURL #SharePoint Site Collection URL

    $Global:fileLocationTemplate = "D:\$RAMSVersion\Templates\" #Template file location
    $Global:fileLocationSiteAsset = "D:\$RAMSVersion\SiteAssets" #Site Asset location, note: no "/" at the end
    $Global:reportSourceDirectory = "D:\$RAMSVersion\Reports" #Report files location
    $Global:fileLocationSitePages = "D:\$RAMSVersion\SitePages"#Site Pages file location
    $Global:fileLocationOnlineHelp = "D:\$RAMSVersion\OnlineHelp"#Online Help file location
    $Global:fileLocationCatalog = "D:\$RAMSVersion\_catalogs"#Catalog file location
    $Global:wfTempFileLocation = "D:\$RAMSVersion\Temp\NotificationWF.csv" #Temp file location
    $Global:diretoryName = "D:\$RAMSVersion\Logfiles"
    $Global:logFileName = "Deploy_All_$env:COMPUTERNAME.log"

    $x = Get-SiteWeb
    $Global:EnvironmentDomainName = $x.Url.Substring(0, $x.Url.IndexOf("sites") - 1)
    # Data Sources connectionstring table
    $Global:rdsConnStringTable = @{
        "RAMS_DB"     = "Data Source=$sqlServerName;Initial Catalog=RAMS" ;
        "RAMS_SP"     = $siteUrl ;
        "DataSource1" = $siteUrl
    }
    ;

    # DataSource Credentials: Username and Password if needed
    $Global:rdsCredentialsTable = @{
        "RAMS_DB" = @($superReaderUserID, $superReaderPwd);
        "RAMS_SP" = @($superReaderUserID, $superReaderPwd)
    }
    ;
    #Schedule task user id and password (farm admin) 
    $Global:STUserName = $farmUserID
    $Global:STPassword = $farmPwd


    $Global:groupList = @(
        "Asheville",
        "Central IRB",
        "Josh Template Testing",
        "Miami",
        "New Orleans",
        "St Louis",
        "LocalSite2" 
        "AO Group"
    ); 

    $Global:scriptInc = Join-Path $fileLocationPS DeploymentDriver.ps1
    echo $scriptInc
    . $scriptInc -siteUrl $siteUrl -fileLocationPS $fileLocationPS -fileLocationTemplate $fileLocationTemplate -fileLocationSiteAsset $fileLocationSiteAsset -reportTargetServerName $reportTargetServerName -reportSourceDirectory $reportSourceDirectory -wfTempFileLocation $wfTempFileLocation -rdsConnStringTable $rdsConnStringTable -rdsCredentialsTable $rdsCredentialsTable  2>&1 > $diretoryName\$logFileName

    #remove old AO and Groups
    Remove-AOGroup 
    #remove couple of pages 
    foreach ($subWeb in (Get-SiteWeb).Webs) {
        $sitepages = $subWeb.Lists["Site Pages"]
        $agendaPage = $sitepages.Items | ? {$_.Name -eq 'AgendaReport.aspx'} 
        if($agendaPage -ne $Null) {
            $agendaPage.Delete() 
            Write-Host ($agendaPage.Name) has been deleted from $subWeb -ForegroundColor Yellow
        }
    }
    $siteAsset = (Get-SiteWeb).Lists["Site Assets"]
    $genAgendaReportJS = $siteAsset.Items | ?{$_.Name -eq 'generateagendareport.js'}
    if($genAgendaReportJS -ne $Null) {
        $genAgendaReportJS.Delete()
        Write-Host ($genAgendaReportJS.Name) has been deleted from (Get-SiteWeb) -ForegroundColor Yellow
    }
} 
#region Deploy RAMS  
function Deploy-RAMS ([string] $Version, [string]$drive) {
    $Global:RAMSVersion = $Version
    if ($env:COMPUTERNAME -match "VAPH" ) {
        $Global:fileLocationPS = $drive + ":\$Global:RAMSVersion\PS\" #PowerShell Script Location
        Set-Location $Global:fileLocationPS
        Copy-Item -path profile.ps1 -Destination C:\Windows\System32\WindowsPowerShell\v1.0
    }            
    #import module, ignore error related to SP snap-in
    if ((Get-Module -Name RAMS_Module_CleanHistory) -ne $null) {
        Remove-Module -Name RAMS_Module_CleanHistory
    }
    if ((Get-Module -Name RAMS_Module_Util) -ne $null) {
        Remove-Module -Name RAMS_Module_Util
    }
    try {
        if ($env:COMPUTERNAME -match "VAPH") {
            Import-Module .\module\RAMS_Module_CleanHistory.psd1 -DisableNameChecking
            Import-Module .\module\RAMS_Module_Util.psd1 -DisableNameChecking
        }
        else {
            Import-Module ..\UtilityScripts\RAMS_Module_CleanHistory.psd1 -DisableNameChecking
            Import-Module ..\UtilityScripts\RAMS_Module_Util.psd1 -DisableNameChecking
        }        
    }
    catch {
        Write-Host Host: $env:COMPUTERNAME is not a Server, SharePoint is not installed on this computer...ignore, information only.
        #do nothing.
    }        
    $Global:hostInfo = Get-RAMSEnv -host $env:COMPUTERNAME
    if($Global:hostInfo -eq $Null) {
        Write-Host No Envrionment match for Host: $env:COMPUTERNAME -ForegroundColor Yellow        
    }
}

#backup current RAMS version
function Backup-RAMS {
    Write-Host "Backing RAMS-SharePoint - Current Version " -ForegroundColor Cyan    
    if((Test-Path D:\spbackup\$env:COMPUTERNAME\SiteCollection) -eq $False) {
        New-Item D:\spbackup\$env:COMPUTERNAME\SiteCollection -ItemType Directory -Verbose
    }
    Backup-SPSite -Identity (Get-SPSite -Identity (Get-SiteURL)) -Path ('D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\rams_sitecollection.bak') -Force -Verbose 
}

#restore last RAMS version
function Restore-RAMS {
    Write-Host "Restoring RAMS-SharePoint - Previous Version " -ForegroundColor Cyan    
    Restore-SPSite -Identity (Get-SiteURL) -Path ('D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\rams_sitecollection.bak') -Force -Verbose
}

function Check-PSVersion {        
    Write-Host $Global:PSVersion.PSVersion.ToString() -ForegroundColor Green
}
    
function Check-RAMSVersion {
    $_uri = Get-SiteURL
    $_html = Invoke-WebRequest -Uri $_uri -UseDefaultCredentials
    $_strongs = $_html.ParsedHtml.getElementsByTagName("strong")
    foreach ($_strong in $_strongs) {
        if ($_strong.innerText.IndexOfAny("v2") -gt 0) {
            Write-Host $_strong.innerText -ForegroundColor Green
            sleep -Seconds 2
        }
    }
}   
   
#endregion
$lastBuild = (ls D:\RAMS-2.3.0\ps\Env.json | Select-Object -Property LastWriteTime).LastWriteTime

if ($env:COMPUTERNAME -match "VAPH") {
    Deploy-RAMS -Version "RAMS-2.3.0" -drive "D"
}
else {
    Deploy-RAMS -Version "RAMS-2.3.0" -drive "C"
}

# Start code block for RAMS Release
$Menu = {
    Write-Host "	**********************************************************" -ForegroundColor White
    Write-Host "	              $RAMSVersion - Installation Script"           -ForegroundColor Cyan
    Write-Host "                                LastBuild: $lastBuild         " -ForegroundColor DarkCyan 
    Write-Host "	**********************************************************" -ForegroundColor White
    Write-Host " "
    Write-Host "	RAMS Installation for SharePoint 2013 on Windows 2012, 2012R2" -ForegroundColor Yellow
    Write-Host " "
    Write-Host "	Please select an option from the list below:" -ForegroundColor White
    Write-Host "	"
    Write-Host "	0) Check PS Version" -ForegroundColor White
    Write-Host "	1) Check RAMS Version" -ForegroundColor White
    Write-Host "	"
    Write-Host "	3) Install New $RAMSVersion" -ForegroundColor Green
    Write-Host "	"
    Write-Host "	4) Clean RAMS History" -ForegroundColor Magenta
    Write-Host "	5) Remove (Non-Zero)Accounts from Site Collection Admin" -ForegroundColor Magenta
    Write-Host "	"
    Write-Host "	6) Backup RAMS (Current Version)" -ForegroundColor Yellow
    Write-Host "	7) Restore RAMS (Previous Version)" -ForegroundColor Yellow
    Write-Host " "
    Write-Host "	100) Restart the Server"  -ForegroundColor Red
    Write-Host "	200) Exit Script" -ForegroundColor Green
    Write-Host "	"
    Write-Host "	**********************************************************" -ForegroundColor White
    Write-Host "	Select an option.. [0-200]? " -ForegroundColor White -nonewline
}

Do { 		
    invoke-command -scriptblock $Menu
    $Choice = Read-Host
    switch ($Choice) {
        0 {
            #Check PS current version
            start-sleep 1
            CLS
            Check-PSVersion		    
            Write-Host " "
            start-sleep 2
        }
        1 {
            #Check RAMS current version
            start-sleep 1
            CLS
            Check-RAMSVersion		    
            Write-Host " "
            start-sleep 2
        }
        2 {
            #Install New PS Version
            start-sleep 1
            CLS
            Install-PS
            Write-Host "Make sure to reboot the server!" -ForegroundColor Magenta
            Write-Host " "
            start-sleep 2
        } 
        3 {
            #Install RAMS Latest version
            start-sleep 1
            CLS
            Install-RAMS
            Write-Host "Make sure to check the Tomcat!" -ForegroundColor Magenta
            Write-Host " "
            start-sleep 2
        }
        4 {
            #clean history
            start-sleep 1
            CLS            
            $_historyDate = Read-Host -Prompt "Please enter history date..(MM/DD/YYYY)" -Verbose            
            Remove-RAMSHistoryData -cleanDate $_historyDate -dbServer $Global:hostInfo.sql
            start-sleep 2
        }        
        5 {
            RemoveNonNzero-RAMS
        }
        6 {
            #backup  - current version
            start-sleep 1
            CLS                        
            Backup-RAMS
            start-sleep 2
        }
        7 {
            #restore  - current version
            start-sleep 1
            CLS                        
            Restore-RAMS
            start-sleep 2
        }
        100 {
            #Exit and restart
            Stop-Transcript
            restart-computer -computername localhost -force
        }
        200 {
            #	Exit            
            Write-Host "Exiting..."
        }
        default {Write-Host "You haven't selected any of the available options. "}
    }
} while ($Choice -ne 200)