﻿try {
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue
}
catch {
 Write-Host "PSSnapin Microsoft.SharePoint.PowerShell already loaded" -ForegroundColor Yellow
}

Set-StrictMode -Version Latest

if($env:COMPUTERNAME -match "VAPH" ) {
    Add-Type -Path .\RAMSUtil.dll
}
else {
    Add-Type -Path ..\UtilityScripts\RAMSUtil.dll
}


function Remove-CalendarColumns ([String[]]$colList, $calTitle) { 
    foreach ($col in $colList) {
        foreach ($subweb in (Get-SiteWeb).Webs) {
             $calList = $subweb.Lists.TryGetList($calTitle)         
             #remove column
             if($calList.Fields.TryGetFieldByStaticName($col) -ne $Null) {
                 $calList.Fields.Delete($col)
                 Write-Host $col has been deleted from $subweb :- $calList  -ForegroundColor Yellow
                 $calList.Update()
             }
        }
    }
}

function RunSQL-ReturnInt ($sqlQuery,$dbServer) {
<#
.DESCRIPTION
The function use Windows Authentication for access to the SQL database, it takes two arguments, both required, please refer the example to get more details.

.EXAMPLE
$cleanUpDate = "'05/30/2017'"
$dbServer = ".DNS           "
$query = "exec DataClenup $cleanUpDate"
$result = RunSQL-ReturnInt -sqlQuery $query -dbServer $dbServer
Write-Host "Number of rows deleted: $result" -ForegroundColor Cyan
#>
    $RAMSSQL = New-Object Gov.VA.RAMS.RAMSUtil.Util    
    $RAMSSQL.SQLConnectionString = "Server=$dbServer;Database=RAMS;Trusted_Connection=Yes;"
    $result = $RAMSSQL.RunSQL($sqlQuery)
    #close connection
    if($RAMSSQL.SQLConnection.State -ne "Closed") {
        $RAMSSQL.CloseSQLConnection()
    }
    return $result
}

function RunSQL-ReturnDR ($sqlQuery,$dbServer) {
<#
.DESCRIPTION
The function use Windows Authentication for access to the SQL database, it takes two arguments, both required, please refer the example to get more details.

.EXAMPLE
 $result = RunSQL-ReturnDR -sqlQuery "select * from Studies" -dbServer DNS  
#>
    $RAMSSQL = New-Object Gov.VA.RAMS.RAMSUtil.Util    
    $RAMSSQL.SQLConnectionString = "Server=$dbServer;Database=RAMS;Trusted_Connection=Yes;"
    $dR = $RAMSSQL.RunSQLResturnDS($sqlQuery)        
    return $dR
}



#region Function Get-RAMSSharePointInstallInfo
Function Get-RAMSSharePointInstallInfo
{
<# 
.EXAMPLE
Get-RAMSSharePointInstallInfo

.EXAMPLE
Get-RAMSSharePointInstallInfo | Format-Table -AutoSize

.EXAMPLE
Get-RAMSSharePointInstallInfo -Verbose

# Requires PowerShell Version 2.0 or Version 3.0
# Requires to be executed as an Administrator
#>
[CmdletBinding()]
Param()

    Begin {
        Write-Verbose "Entering Begin Block"

        if (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
        {
            Write-Warning "You do not have Administrator rights to run this script!`nPlease re-run this script as an Administrator!"
            Exit
        }
        else {
            Write-Verbose "You are an Administrator. Continue to Process Block."
        }


        try {
            $isSPPowerShellLoaded = $null
            $SPSnapin = "Microsoft.SharePoint.PowerShell"
            if (Get-PSSnapin $SPSnapin -ErrorAction SilentlyContinue) {
            	$isSPPowerShellLoaded = $true
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin already registered and loaded"
            }
            elseif (Get-PSSnapin $SPSnapin -Registered -ErrorAction SilentlyContinue) {
                Add-PSSnapin $SPSnapin
            	$isSPPowerShellLoaded = $true 
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin is registered and has been loaded for script operation"
            }
            else {
		        $isSPowerShellLoaded = $false
                Write-Warning "Microsoft.SharePoint.PowerShell Snappin not found."
		        Write-Warning "Will not be able to report SharePoint Build Version"
            }

        }
        catch {
	        $isSPPowerShellLoaded = $false
            Write-Error "There was an issue loading the Microsoft.SharePoint.PowerShell Snappin. Script execution will continue"
	        Write-Error $_
	        Write-Warning "Will not be able to report SharePoint Build Version"            
        }

        Write-Verbose "Leaving Begin Block"
    }

    Process {
        Write-Verbose "Entering Process Block"
        
        Write-Verbose "Define SharePoint 2010 Products Registry Path variable"
        $RegPath2010 = "HKLM:\SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\14.0\WSS\InstalledProducts\"

        Write-Verbose "Define SharePoint 2010 Products hash table"
        Write-Verbose "SharePoint 2010 SKU ID's found at http://msdn.microsoft.com/en-us/library/ff721969(v=office.14).aspx"
        $SP2010Products = @{
				            "BEED1F75-C398-4447-AEF1-E66E1F0DF91E" = "SharePoint Foundation 2010"
				            "1328E89E-7EC8-4F7E-809E-7E945796E511" = "Search Server Express 2010"
				            "B2C0B444-3914-4ACB-A0B8-7CF50A8F7AA0" = "SharePoint Server 2010 Standard Trial"
				            "3FDFBCC8-B3E4-4482-91FA-122C6432805C" = "SharePoint Server 2010 Standard"
				            "88BED06D-8C6B-4E62-AB01-546D6005FE97" = "SharePoint Server 2010 Enterprise Trial"
				            "D5595F62-449B-4061-B0B2-0CBAD410BB51" = "SharePoint Server 2010 Enterprise"
				            "BC4C1C97-9013-4033-A0DD-9DC9E6D6C887" = "Search Server 2010 Trial"
				            "08460AA2-A176-442C-BDCA-26928704D80B" = "Search Server 2010"	
				            "84902853-59F6-4B20-BC7C-DE4F419FEFAD" = "Project Server 2010 Trial"
				            "ED21638F-97FF-4A65-AD9B-6889B93065E2" = "Project Server 2010"
				            "926E4E17-087B-47D1-8BD7-91A394BC6196" = "Office Web Companions 2010"
			                }


        Write-Verbose "Define SharePoint 2013 Products Registry Path variable"
        $RegPath2013 = "HKLM:\SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\15.0\WSS\InstalledProducts\"

        Write-Verbose "Define SharePoint 2013 Products hash table"
        Write-Verbose "SharePoint 2013 SKU ID's found at http://msdn.microsoft.com/en-us/library/jj659075.aspx"
        $SP2013Products =  @{
				            "35466B1A-B17B-4DFB-A703-F74E2A1F5F5E" = "Project Server 2013"
				            "BC7BAF08-4D97-462C-8411-341052402E71" = "Project Server 2013 Preview"
				            "C5D855EE-F32B-4A1C-97A8-F0A28CE02F9C" = "SharePoint Server 2013"
				            "CBF97833-C73A-4BAF-9ED3-D47B3CFF51BE" = "SharePoint Server 2013 Preview"
				            "B7D84C2B-0754-49E4-B7BE-7EE321DCE0A9" = "SharePoint Server 2013 Enterprise"
				            "298A586A-E3C1-42F0-AFE0-4BCFDC2E7CD0" = "SharePoint Server 2013 Enterprise Preview"
				            "D6B57A0D-AE69-4A3E-B031-1F993EE52EDC" = "Microsoft Office Web Apps Server 2013"
				            "9FF54EBC-8C12-47D7-854F-3865D4BE8118" = "SharePoint Foundation 2013"	
			                }


        Write-Verbose "Define function variables to store data based on current environment"
        $RegPath = $null
        $SPProducts = $null
        $SharePointVersion = $null
        $CanProcess = $false

        Write-Verbose "Test Registry Paths to determine if current environment has SharePoint 2010 or 2013 installed"
        $is2010 = Test-Path $RegPath2010
        $is2013 = Test-Path $RegPath2013

        Write-Verbose "Populate function variables with SharePoint version specific data, if the environment contains SharePoint 2010/2013"
        if($is2010) {
            Write-Verbose "SharePoint 2010 is installed"
            $RegPath = $RegPath2010
            $SPProducts = $SP2010Products
            $SPVersion = 2010
            $CanProcess = $true

        }
        elseif($is2013) {
            Write-Verbose "SharePoint 2013 is installed"
            $RegPath = $RegPath2013
            $SPProducts = $SP2013Products
            $SPVersion = 2013
            $CanProcess = $true

        }
        else {
            Write-Warning "SharePoint 2010 or 2013 is not installed within current environment"   
            Write-Warning "Hash table intended to store SharePoint Product information will be empty"    
        }


        Write-Verbose "Define hash table to store installed SharePoint Product information"
        $InstalledProducts = @{}

        Write-Verbose "Verify that current environment is hosting SharePoint 2010 or 2013"
        if($CanProcess) {

        	Write-Verbose "Determine whether the Microsoft.SharePoint.PowerShell Snappin is loaded"
	        if($isSPPowerShellLoaded) {
		        try {
			        Write-Verbose "Get SharePoint Build Version"
			        $SPBuildVersion = (Get-SPFarm).BuildVersion.ToString()

			        Write-Verbose "Adding the SharePoint Build Version to the InstalledProducts hash table"
			        $InstalledProducts.Add("SharePoint $SPVersion Build Version",$SPBuildVersion)
		        }
		        catch {
			        Write-Error "Unable to get the SharePoint Build Version."
			        Write-Error $_
		        }
	        }


            Write-Verbose "SharePoint $SPVersion is installed within the current environment"
            Write-Verbose "Populate InstalledProducts hash table with SharePoint Products installed on current environment"
            Get-Item $RegPath | 
                Select-Object -ExpandProperty property | 
                ForEach-Object {(Get-ItemProperty -Path $RegPath -Name $_).$_}	| 
                    ForEach-Object {$InstalledProducts.Add($SPProducts.Get_Item($_),$_)}
        }

        Write-Verbose "Output the InstalledProducts hash table to pipeline."
        Write-Output $InstalledProducts 
        
        Write-Verbose "Leaving Process Block"       
    }

}

#endregion

function Get-SiteURL {
    $site = Get-SPSite -Limit All | Select-String IRBApplications
    $x = $site.ToString();
    $url = $x.Substring($x.IndexOf("=")+1)
    $w = Get-SPWeb $url
    return $url
}

function Get-SiteWeb {
    $url = Get-SiteURL
    $xWeb = Get-Spweb $url
    return $xWeb
}


Function Deploy-Feature {

Param(          [Parameter(Mandatory=$True)]
                [string]$SiteCollection,

                [Parameter(Mandatory=$True)]
                [string]$site,

                [Parameter(Mandatory=$True)]
                [string]$FeatureName,

                [Parameter(Mandatory=$True)]
                [string]$OldFeatureId,

                [Parameter(Mandatory=$True)]
                [string]$FeatureId,

                [Parameter(Mandatory=$True)]
                [string]$file,

                [Parameter(Mandatory=$True)]
                [bool] $bolUninstall,

                [Parameter(Mandatory=$False)]
                [bool] $bolEnableFeatureOnly   
                )
    

        Add-PSSnapin Microsoft.SharePoint.PowerShell

    # GET THE SITE COLLECTION
    $SiteCollectionweb = Get-SPWeb $SiteCollection

    # GET THE SITE
    $web = Get-SPWeb $site

    # CLEAN and old one first
    if($bolEnableFeatureOnly -eq $False) {
        if($bolUninstall)
        {
            Write-Host "  Uninstall old ones"
            
            Write-Host "    Disabling feature Identity $OldFeatureId URL $web.Url"
            Disable-SPFeature -Identity $OldFeatureId -Url $SiteCollectionweb.Url -Force -ErrorAction SilentlyContinue  
            
            Write-Host "    Uninstalling feature Identity $FeatureName Site $SiteCollectionweb.Url"
            Uninstall-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Site 
            
            Write-Host "    Removing feature Identity $FeatureName Site $SiteCollectionweb.Url"
            Remove-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Site 
            $web.Update()     
            
            #remove from workflow manager as well
            Remove-WorkFlow -wfMatchName ($FeatureName.Replace(".wsp","")) -wfWeb $web
        }
        # ADD/UPLOAD TO THE SOLUTIONS
        Write-Host "  Adding new ones"
        Write-Host "    Adding feature file $file Site $SiteCollectionweb.Url"
        Add-SPUSerSolution -LiteralPath $file -Site $SiteCollectionweb.Url -Verbose 
        $web.Update()
        # ACTIVATE THE SOLUTION
        Write-Host "  Activating new ones"
        Write-Host "    Install feature Identity $FeatureName Site $SiteCollectionweb.Url"
        Install-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Url 
        $web.Update()
    }
    # ENABLE THE FEATURE FOR THE GIVEN SITE
    # feature id goes into -Identity
    Write-Host "  Enabling new ones"
    Write-Host "    Enabling feature Identity $FeatureName Url $web.Url"
    Enable-SPFeature -Identity $FeatureId -Url $web.Url -Verbose -Force
    $web.Update()
}

function Add-SiteColumnToCTByTitle ([Microsoft.SharePoint.SPSecurableObject]$web, [String]$ctName,[String]$sFieldTitle) {
    $ct = $web.ContentTypes[$ctName];
    $field=$web.Fields.GetField($sFieldTitle)
    $sFieldStaticName = $field.StaticName

    if(!$ct.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
	    $link = new-object Microsoft.SharePoint.SPFieldLink $field
        #check if comments fields exist or not.
        if(($ct.FieldLinks | Where-Object {$_.Name -eq $field.InternalName}) -EQ $null) {
	        $ct.FieldLinks.Add($link)
	        $ct.Update($true)
            Write-Host $sFieldTitle Field has been added to $ct -ForegroundColor Green
        }
        else {
            Write-Host $sFieldTitle Field already exist -ForegroundColor Yellow
        }
    }
}


function Add-SiteColumnToCT ([Microsoft.SharePoint.SPSecurableObject]$web, [String]$ctName,[String]$sFieldInternalName, [String]$sFieldStaticName) {
    $ct = $web.ContentTypes[$ctName];
    if(!$ct.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
	    $field=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
	    $link = new-object Microsoft.SharePoint.SPFieldLink $field
        #check if comments fields exist or not.
        if(($ct.FieldLinks | Where-Object {$_.Name -eq $sFieldInternalName}) -EQ $null) {
	        $ct.FieldLinks.Add($link)
	        $ct.Update($true)
            Write-Host $sFieldInternalName Field has been added to $ct -ForegroundColor Green
        }
        else {
            Write-Host $sFieldInternalName Field already exist -ForegroundColor Yellow
        }
    }
}


function Set-SiteColumn([Microsoft.SharePoint.SPSecurableObject]$web, [String]$sFieldDisplayName,[String]$sFieldStaticName, [String]$sFieldInternalName, [Microsoft.SharePoint.SPFieldType]$sFieldType, [System.Collections.Specialized.StringCollection] $sChoice, [String]$spFieldGroup, [switch] $isRequired, [switch] $showInEditForm, [switch] $showInNewForm) {
    $web = (Get-SiteWeb)
    if (!$web.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
	    $sFieldDisplayName=$sFieldDisplayName 
	        write-Host "Adding the field $sFieldDisplayName to the Site" -foregroundcolor cyan   
            $sFieldStaticName = $web.Fields.Add($sFieldInternalName,$sFieldType, $false, $false, $sChoice);
	        $spField=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
	        $spField.Group= $spFieldGroup 
	        $spField.Title=$sFieldDisplayName 
            
            if($isRequired) {$spField.Required = $True}
            if($showInEditForm) {$spField.ShowInEditForm = $True}
            if($showInNewForm) {$spField.ShowInNewForm = $True}

	        $spField.Update()  
    }
    else {
         Write-Host $sFieldInternalName Field already exist -ForegroundColor Yellow
    }
}
function Set-SiteColumnLookup([Microsoft.SharePoint.SPSecurableObject]$web, [String]$sFieldStaticName, [String]$sFieldDisplayName, [String]$lookupListTitle, [String]$spFieldGroup) {
    if (!$web.Fields.ContainsFieldWithStaticName($sFieldStaticName)){	    
	        write-Host "Adding the field $sFieldDisplayName to the Site" -foregroundcolor cyan               
	        $sFieldStaticName = $web.Fields.AddLookup($sFieldDisplayName,(($web.Lists.TryGetList($lookupListTitle)).Id), $false);            
	        $spField=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
	        $spField.Group= $spFieldGroup 
	        $spField.Title=$sFieldDisplayName 
	        $spField.Update()  
    }
    else {
         Write-Host $sFieldInternalName Field already exist -ForegroundColor Yellow
    }
}

function Update-SiteColumnLookup ($siteColumnTitle,$lookupListTitle,$lookupListFieldTitle){
    $siteColumnFld = ($webSite.Fields | ? {$_.Title -match $siteColumnTitle})
    $listLookup = $webSite.Lists.TryGetList($lookupListTitle)
    $siteColumnFld.LookupList = $listLookup.ID
    $siteColumnFld.LookupField = $lookupListFieldTitle
    $siteColumnFld.Update()
}

#region Add-FacilityWorkflow
function Add-FacilityWorkflow ($oldFeatureName, $featureTemplateName, $newFeatureId, $searchDisplay, $listName, $unInstall)
{
    $bolEnableFeatureOnly = $False #initialize
   	# GET THE SITE
    $_siteURL = Get-SiteURL
    $w = Get-SPWeb $_siteURL

    Write-Host ([String]::Format("Procesing web {0}",$w.Url)) -foregroundcolor Cyan            

    if($w.Webs.Count -gt 0)
    {
        foreach($web in $w.Webs)
        {   
            #Enable old workflows from 2.1 or before
            if($oldFeatureName -contains "*IACUC*" -or $oldFeatureName -contains "*RDC*") {            
                Enable-SPFeature -Identity $oldFeatureName -Url $web.Url -Verbose -Force
                Enable-SPFeature -Identity $oldFeatureName -Url $siteUrl -Verbose -Force
            }
            $web.Update()
            #get above old workflows
            $Oldfeature = $null #initialize

            if($bolEnableFeatureOnly -eq $False) {
                try {
                $Oldfeature = Get-SPFeature -Identity $oldFeatureName -Web $_siteURL -ErrorAction Ignore -Verbose
                }
                catch {
                    Write-Host "Feature: $oldFeatureName does not exist for Site: $web.Url"
                }
            }
            $tempFeatureId = $null
            if($Oldfeature -ne $null) {
                $tempFeatureId = $Oldfeature.Id             
            }
            if ($tempFeatureId -eq $null)
            {
                $tempFeatureId = $newFeatureId
            }
            Write-Host "Calling Deploy-Feature on $web with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId "
            if($bolEnableFeatureOnly -eq $False) {
            	Deploy-Feature -SiteCollection $_siteURL -site $web.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $bolEnableFeatureOnly
                $bolEnableFeatureOnly = $True
            }
            else {
                Deploy-Feature -SiteCollection $_siteURL -site $web.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $bolEnableFeatureOnly
            }
        }
    }
}
#endregion 

#region Add Site Collection level workflow
function Add-SiteCollectionWorkFlow ($featureTemplateName, $newFeatureId, $oldFeatureId, $searchDisplay, $unInstall) {
 try
    {    
   	    # GET THE SITE
        $_siteURL = Get-SiteURL
        $w = Get-SPWeb $_siteURL
        Write-Host ([String]::Format("Procesing web {0}",$w.Url)) -foregroundcolor Cyan            
     
        #$featureTemplateName="Notification Workflow Configuration Workflow.wsp"
        #$newFeatureId="df439192-e32f-4760-9bef-d239d8519593"
        
        $tempFeatureId = $Null
        $Oldfeature = Get-SPFeature -Limit ALL  -Web $w.Url | Where-Object {$_.DisplayName -like $searchDisplay}           

        if($Oldfeature -ne $Null) {
            $tempFeatureId = $Oldfeature.Id
        }
        if ($tempFeatureId -eq $null)
        {
            $tempFeatureId = $oldFeatureId
        }
      	Write-Host "Calling Deploy-Feature on $w with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId "
        Deploy-Feature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $False	 

    }
    catch
    {
        Write-Host ([String]::Format("Error processing web at $_siteURL, with Exception: {0}", $_.Exception.Message)) -foregroundcolor Red
    }  
}
#endregion

#region utility functions Get-DateFromDays, Format-ListOutput, Get-ListInternalName
function Get-DateFromDays ($minusdays) {
    $days = $minusdays 
    $count = $collListItems.Count;
    $date = get-date;
    return $date.AddDays($days);

}

function Format-ListOutput($listTable) {
  $listTable | Format-List
}

function Get-ListInternalName ($staticName, $list) {
    $list.Fields.TryGetFieldByStaticName($staticName).InternalName
    return;
}
function Get-SPList($web, $Name, $Url, $subWeb) {
   $list = $null
    try {  
        if($subWeb -eq $null) {  
            $list = $web.GetListFromURL($Url) 
         }
         else {
            $list = $subWeb.GetListFromURL($Url) 
         }
        return $list
    }
    catch {
       try {
        if($subWeb -eq $null) {  
            $list = $web.GetList($Name) 
         }
         else {
            $list = $subWeb.GetList($Name) 
         }
         return $list
        }
            catch {  
          return $list          
        }
    }   
}


function Remove-DataFromAList ($Site, $subSite, $filterDate, $listUrl, $listName)
{
    #get a collection of ids
    $itemsToDelete = New-Object System.Collections.ArrayList
    $list = Get-SPList -web $Site -Name $listName -Url $listUrl -subWeb $subSite
    #get application list
    if($list -ne $null)
    {
        #Get all items in this list and save them to a variable
        $items = $list.items
        $itemCount = 0
        #Go through all items
        Write-Host Total Items in the $list.EntityTypeName : $items.Count -ForegroundColor Green
        foreach($item in $items)
        {
            if($item["Created"] -lt $filterDate)
            { 
                #add each item id to the delete list  
                Write-Host Add item# $itemCount  from the list for Study#: $item["Title"] on $item["Created"] -ForegroundColor Cyan                                 
                $itemsToDelete.Add($item.ID)                
                $itemCount ++;
            }            
        }        
        #now start deleting those items
        if($itemCount -eq 0) {
            Write-Host Nothing to delete, count: $itemCount  -ForegroundColor Yellow
         return
        }
        Write-Host Total items to be deleted $itemCount  -ForegroundColor Yellow
        foreach($itemToDelete in $itemsToDelete) {
            Write-Host Deleting item# $itemToDelete -ForegroundColor Cyan  
            $items.DeleteItemById($itemToDelete);                                                                                                    
        }
        $items.Update();
        $list.Update();
        Write-Host Total Items After $list.items.Count  -ForegroundColor Green        
    }
    else
    {
        Write-Host No list found by the name: $listUrl
    }
}
#endregion

#region Module Manifest
function Update-UtilMF {
    New-ModuleManifest -Path .\RAMS_Module_Util.psd1 -ModuleVersion 1.0.0 -Author "Chita Nilak, ByLight" -RootModule .\RAMS_Module_Util.psm1 -Description "RAMS Utility Functions" -CompanyName "VA.GOV.RAMS" -Verbose 
}
#endregion

#region 
#New-RAMSEnvironment
function Get-RAMSEnv ($host) {
    $_host = $host
    $_env = Get-Content .\Env.json | ConvertFrom-Json
    $_server = $null
    switch ($_host) {
        $_env.DEV.sp {$_server = $_env.DEV}    
        $_env.TEST.sp {$_server = $_env.TEST}    
        $_env.PPD.sp {$_server = $_env.PPD}    
        $_env.SQA.sp {$_server = $_env.SQA}    
        $_env.PROD.sp {$_server = $_env.PROD}    
    }
    return $_server
}

function New-RAMSEnv {

    $_serverEnv = @{
        DEV = @{
                            "sp" = "DNS  ";						    
                            "sql" = "DNS  ";
						    "ssrs" = "DNS  ";
						    "superReader" = @("",""); #READ-ONLY access to RAMS-DB
						    "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
					    }; #DEV
        TEST = @{
                            "sp" = "DNS  ";						    
						    "sql" = "DNS  ";
						    "ssrs" = "DNS  ";
						    "superReader" = @("",""); #READ-ONLY access to RAMS-DB
						    "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
					    }; #TEST
        SQA = @{
                            "sp" = "DNS  ";						    
						    "sql" = "DNS";
						    "ssrs" = "DNS";
						    "superReader" = @("",""); #READ-ONLY access to RAMS-DB
						    "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
					    }; #SQA
        PPD = @{
                            "sp" = "DNS  ";						    
						    "sql" = "DNS";
						    "ssrs" = "DNS";
						    "superReader" = @("",""); #READ-ONLY access to RAMS-DB
						    "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
					    }; #PPD
        PROD = @{
                            "sp" = "DNS  APPRAM20";						    
						    "sql" = "DNS";
						    "ssrs" = "DNS";
						    "superReader" = @("",""); #READ-ONLY access to RAMS-DB
						    "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
					    }; #PROD
    }
    if((Get-ChildItem ".\" -Filter "ENV.json") -ne $null) { 					
        Remove-Item .\ENV.json -Verbose
    }
    ConvertTo-json $_serverEnv -Compress | New-Item -Name ENV.json -Path .\ -ItemType file -Force
    return Get-Content .\Env.json | ConvertFrom-Json
}
function Get-FacilitySites {
    $webs = Get-SiteWeb 
    $_facilities = New-Object System.Collections.ArrayList    
    foreach($web in $webs.Webs) {        
       $_facilities.Add($web.Name)       
    }
    #return
    $_facilities
}
function New-RAMSGroup ($facility, $newGroupName, $newGroupPermission, $groupOwnerSuffix, $newGroupDescSuffix) {
    
    #Get site and Group objects  
    $_url = Get-SiteURL 
    $_Site = Get-SPSite -Identity $_url
    
    #add new security group
    $_siteGroups = $_Site.RootWeb.SiteGroups
    $_ownerGroup = $_siteGroups["$facility $groupOwnerSuffix"]
    
    #check if the group already exist
    try {
        $_siteGroups.Add("$facility $newGroupName",$_ownerGroup,$null,"$facility $newGroupDescSuffix")
    }    
    catch {
        $_siteGroups.Remove("$facility $newGroupName")
        $_siteGroups.Add("$facility $newGroupName",$_ownerGroup,$null,"$facility $newGroupDescSuffix")
    }
    $_Site.RootWeb.Update()

    #Change AO group permission at site collection level to EDIT
    $role = $_Site.RootWeb.RoleDefinitions[$newGroupPermission]
    $_newGroup = $_Site.RootWeb.SiteGroups["$facility $newGroupName"]
    $RoleAssignment = New-Object Microsoft.SharePoint.SPRoleAssignment($_newGroup)
    $RoleAssignment.RoleDefinitionBindings.Add($role)

    # the following could be for a web, list or item object
    $_Site.RootWeb.RoleAssignments.Add($RoleAssignment)
    $_Site.RootWeb.Update()
    Write-Host """$facility $newGroupName"" has been added w/ ""$newGroupPermission"" permission on $_Site" -ForegroundColor yellow
    
    #dispose site object   
    $_Site.Dispose();  
}
function Add-FacilityGroup ($newGroupName, $newGroupPermission) {
    $_facilityWebs = Get-SiteWeb
    $_facilityList = $_facilityWebs.GetListFromUrl("Lists/Facility/AllItems.aspx")

    #add each new groups to the respective facility sub-site.
    foreach ($item in $_facilityList.Items) {
        #facility sub-site
        $facilitySubsite = $item["Facility Subsite"]
        #facility name
        $facilityName = $item["Facility Name"]
        #foreach
        foreach ($facilityWeb in $_facilityWebs.Webs) {
            if($facilitySubsite.IndexOf($facilityWeb.Name) -gt 0) {
                #get role
                $role = $facilityWeb.RoleDefinitions[$newGroupPermission]
                $_newGroup = $_facilityWebs.SiteGroups["$facilityName $newGroupName"]
                $RoleAssignment = New-Object Microsoft.SharePoint.SPRoleAssignment($_newGroup)
                $RoleAssignment.RoleDefinitionBindings.Add($role)

                # the following could be for a web, list or item object
                $facilityWeb.RoleAssignments.Add($RoleAssignment)
                $facilityWeb.Update()
                Write-Host """$_newGroup"" has been added w/ ""$newGroupPermission"" permission on $facilityWeb" -ForegroundColor yellow            
            
                #$item["Research Compliance Officers"]        
            }
            $facilityWeb.Update()
        }    
    }
}

function Remove-WorkFlow ([string] $wfMatchName, $wfWeb) {
    $wfm = New-Object Microsoft.SharePoint.WorkflowServices.WorkflowServicesManager($wfWeb);

    $wfss =$wfm.GetWorkflowSubscriptionService()

    $wfis=$wfm.GetWorkflowInstanceService()

    #$wfss.EnumerateSubscriptions()

    foreach($item in $wfss.EnumerateSubscriptions()){        
        if($item.Name -match $wfMatchName){
            $wfss.DeleteSubscription($item.Id)
            Write-Host ($item.Name) WorkFlow has been removed from $wfWeb -ForegroundColor Yellow
        }
    }
    $wfWeb.Update()    
}
function Rename-List($spWeb,$listName,$newTitle) {
    ForEach($culture in $spWeb.SupportedUICultures)
    {
        [System.Threading.Thread]::CurrentThread.CurrentUICulture=$culture;
        #check if the list has already been named or not
        if(($spWeb.Lists.TryGetList($newTitle)) -eq $null) {
            $list = $spWeb.Lists.TryGetList($listName) 
            $list.Title = $newtitle;                    
            $list.RootFolder.MoveTo($newtitle)
            $list.Update();
        }        
    }
    $spWeb.Dispose()
}

function Clone-Calendar($spWeb, $calName,$OldCalName, $calTitle) 
{  
    $SPTemplate = (Get-SiteWeb).ListTemplates["Calendar"]
    
    $newCal = $spWeb.Lists.TryGetList($calTitle)
    #remove if the clone list already exist
    if($newCal -ne $null) {    
        $spWeb.Lists.Delete($newCal.ID)
        $spWeb.Update()
    }
    #create a new clone list
    $spWeb.Lists.Add($calTitle,$calTitle,$SPTemplate)
    $spWeb.Update()
    
    #get old list (after rename)    
    $oldList = $spWeb.Lists.TryGetList($OldCalName)

    #get new list 
    $newList = $spWeb.Lists.TryGetList($calTitle)
               
    if($oldList -ne $null)
    {
        
        foreach($oldFld in $oldList.Fields) {            
            if(($newList.Fields.ContainsFieldWithStaticName($oldFld.StaticName) -eq $False) -and 
                   ($newList.Fields.ContainsField($oldFld.Title) -eq $False))
            {          
		        $newList.Fields.Add($oldFld);
                $newList.Update()
                Write-Host $oldFld has been added to $newList -ForegroundColor Green
            }
            else {
                Write-Host $oldFld already exist in $newList -ForegroundColor Yellow
            }
        }
        
        $spWeb.Update()
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };

        <#add old view
        $oldView = $oldList.Views | ? { $_.Title -eq 'All Events' };
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };
        #clone view fields
        foreach($oldViewFld in $oldView.ViewFields){
            if($newView.ViewFields.exists($oldViewFld)) {
                Write-Host ($oldViewFld) already exist in $newView -ForegroundColor Yellow
            }
            else {
                $newView.ViewFields.Add($oldViewFld)
                Write-Host ($oldViewFld) has been added to $newView -ForegroundColor Green
            }
        }
        #>
        $contentType = "Event"
        $newFieldOrder = @("Title", "EventDate", "EndDate", "Location", "fAllDayEvent", "fRecurrence","EventAttendees", "EventCommitteeAttendees",  "ParticipantsPicker","Administrative_x0020_Items", "TaxKeyword", "ApprovedApplicationLookup", "IACUCMeetingMinuteLookup", "RDCMeetingMinuteLookup", "Decision", 
        "Deliberation"
        )
	    $ct = $newList.ContentTypes[$contentType] 	    
	    $ct.FieldLinks.Reorder($newFieldOrder)
	    $ct.Update() 
	    $newList.Update()
        #set as default view
        $newView.DefaultView = $true
        $newView.Update()
        #migrate data from old list to new list
        $newItem = $null
        foreach($oldItem in $oldList.Items) {
            $newItem = $newList.AddItem()
            foreach($fld in $oldItem.Fields) {
              if(($newList.Fields.ContainsField($fld.Title) -eq $True)) {  
                if("ID","WF","Link","_","Agenda","Attachments","Facilities" -notcontains $fld.StaticName){
                    if($fld.ReadOnlyField -eq $False) {
                        $newItem[$fld.StaticName] = $oldItem[$fld.StaticName]
                        Write-Host Item: $newItem[$fld.StaticName] has been added to the $newList -ForegroundColor Cyan            
                    }              
                 } 
               } 
             }
            $newItem.Update()            
        }
        $newList.Update()
    }
}
function Remove-SitePages ([string[]]$sitePages) {
    foreach($page in $sitepages) {
        $targetPage = ((Get-SiteWeb).Lists["Site Pages"]).Items | ? {$_.Name -eq $page} 
        if($targetPage -ne $Null) {
            $targetPage.Delete() 
            Write-Host $page has been deleted from Site Collection: (Get-SiteWeb) -ForegroundColor Yellow
        }
    }
}
function Remove-SiteAssets ([string[]]$siteAssets) {
    foreach($asset in $siteassets) {
        $targetAsset = ((Get-SiteWeb).Lists["Site Assets"]).Items | ? {$_.Name -eq $asset} 
        if($targetAsset -ne $Null) {
            $targetAsset.Delete() 
            Write-Host $asset has been deleted from Site Collection: (Get-SiteWeb) -ForegroundColor Yellow
        }
    }
}

function Remove-FacilitySitePages ([string[]]$sitePages) {
    foreach ($subWeb in (Get-SiteWeb).Webs) {        
        foreach($page in $sitepages) {
            $targetPage = ($subWeb.Lists["Site Pages"]).Items | ? {$_.Name -eq $page} 
            if($targetPage -ne $Null) {
                $targetPage.Delete() 
                Write-Host ($targetPage.Name) has been deleted from $subWeb -ForegroundColor Yellow
            }
        }
    }
}
function Remove-FacilitySiteAssets ([string[]]$siteAssets) {
    foreach ($subWeb in (Get-SiteWeb).Webs) {
        foreach($asset in $siteassets) {
            $targetAsset = ($subWeb.Lists["Site Assets"]).Items | ? {$_.Name -eq $asset} 
            if($targetAsset -ne $Null) {
                $targetAsset.Delete() 
                Write-Host ($targetAsset.Name) has been deleted from $subWeb -ForegroundColor Yellow
            }
        }
    }
}
function New-SpListView ([System.Collections.Specialized.StringCollection] $fldList, $viewTitle, $listName,$viewQuery,$viewRowLimit) {

    #Get destination site and list
    $web = Get-SPWeb -Identity (Get-SiteURL)    
    $list = $web.GetList($web.ServerRelativeUrl.TrimEnd("/") + "/Lists/$listName")
    
    #Title property
    $viewTitle = $viewTitle
    $View = $list.Views[$viewTitle]
    if ($View -ne $null) {
           $list.Views.Delete($View.ID)
           $list.Update()
    }
        #Add the column names from the ViewField property to a string collection
    $viewFields = New-Object System.Collections.Specialized.StringCollection
    foreach ($fldName in $fldList) {
        $viewFields.Add($fldName) > $null
    }

    #Paged property
    $viewPaged = $true

    #DefaultView property
    $viewDefaultView = $false

    #Create the view in the destination list
    $newview = $list.Views.Add($viewTitle, $viewFields, $viewQuery, $viewRowLimit, $viewPaged, $viewDefaultView)
    Write-Host ("View '" + $newview.Title + "' has been added to the list '" + $list.Title + "' on site " + $web.Url) -ForegroundColor Green

    #update view and list
    $newview.Update()
    $list.Update()    
}

function Update-SpListView ([System.Collections.Specialized.StringCollection] $fldList, $listName) {
    #Get destination site and list
    $web = Get-SPWeb -Identity (Get-SiteURL)    
    $list = $web.GetList($web.ServerRelativeUrl.TrimEnd("/") + "/Lists/$listName")
    
    #Update Views
    try {
        foreach($View in $list.Views) {
            #Add the column names from the ViewField property to a string collection
            $viewFields = $View.ViewFields
            foreach ($fldName in $fldList) {
                $ViewFields.Add($fldName) > $null
            }    
            #update view and list        
            $View.Update()
            Write-Host ("View '" + $View.Title + "' has been updated for the list: '" + $list.Title + "' on site " + $web.Url) -ForegroundColor Green
       }
   }
   catch {
    #do nothing.
   }
   finally {
    $list.Update()
   }
}

Function Remove-ContentTypeFromList()
{
    param
    (
        [Parameter(Mandatory=$true)] [string] $SiteURL,
        [Parameter(Mandatory=$true)] [string] $ListName,
        [Parameter(Mandatory=$true)] [string] $ContentTypeName
    )
 
    Try {

        Add-Type -Path "C:\Program Files\Common Files\Microsoft Shared\Web Server Extensions\15\ISAPI\Microsoft.SharePoint.Client.Runtime.dll"
        Add-Type -Path "C:\Program Files\Common Files\Microsoft Shared\Web Server Extensions\15\ISAPI\Microsoft.SharePoint.Client.dll"

        $Credentials = New-Object System.Net.NetworkCredential("AI","AI","DNS      ")
         
        #Setup the context
        $Ctx = New-Object Microsoft.SharePoint.Client.ClientContext($SiteURL)
        $Ctx.Credentials = $Credentials
     
        #Get the List
        $List = $Ctx.Web.Lists.GetByTitle($ListName)
        $Ctx.Load($List)
 
        #Get the content type from list
        $ContentTypeColl = $List.ContentTypes
        $Ctx.Load($ContentTypeColl)
        $Ctx.ExecuteQuery()
 
        #Get the content type to remove
        $CTypeToRemove = $ContentTypeColl | Where {$_.Name -eq $ContentTypeName}
        If($CTypeToRemove -ne $Null)
        {
            #Remove content type from list
            $CTypeToRemove.DeleteObject()
            $Ctx.ExecuteQuery()
 
            Write-host "Content Type '$ContentTypeName' Removed From '$ListName'" -f Green
        }
        else
        {
            Write-host "Content Type '$ContentTypeName' doesn't exists in '$ListName'" -f Yellow
            Return
        }
   }
    Catch {
        write-host -f Red "Error Removing Content Type from List!" $_.Exception.Message
    }
}


function Copy-SpList($spWeb, $calName,$OldCalName, $calTitle) 
{  
    $SPTemplate = (Get-SiteWeb).ListTemplates["Calendar"]
    
    $newCal = $spWeb.Lists.TryGetList($calTitle)
    #remove if the clone list already exist
    if($newCal -ne $null) {    
        $spWeb.Lists.Delete($newCal.ID)
        $spWeb.Update()
    }
    #create a new clone list
    $spWeb.Lists.Add($calTitle,$calTitle,$SPTemplate)
    $spWeb.Update()
    
    #get old list (after rename)    
    $oldList = $spWeb.Lists.TryGetList($OldCalName)

    #get new list 
    $newList = $spWeb.Lists.TryGetList($calTitle)
               
    if($oldList -ne $null)
    {
        
        foreach($oldFld in $oldList.Fields) {            
            if(($newList.Fields.ContainsFieldWithStaticName($oldFld.StaticName) -eq $False) -and 
                   ($newList.Fields.ContainsField($oldFld.Title) -eq $False))
            {          
		        $newList.Fields.Add($oldFld);
                $newList.Update()
                Write-Host $oldFld has been added to $newList -ForegroundColor Green
            }
            else {
                Write-Host $oldFld already exist in $newList -ForegroundColor Yellow
            }
        }
        
        $spWeb.Update()
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };

        <#add old view
        $oldView = $oldList.Views | ? { $_.Title -eq 'All Events' };
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };
        #clone view fields
        foreach($oldViewFld in $oldView.ViewFields){
            if($newView.ViewFields.exists($oldViewFld)) {
                Write-Host ($oldViewFld) already exist in $newView -ForegroundColor Yellow
            }
            else {
                $newView.ViewFields.Add($oldViewFld)
                Write-Host ($oldViewFld) has been added to $newView -ForegroundColor Green
            }
        }
        #>
        $contentType = "Event"
        $newFieldOrder = @("Title", "EventDate", "EndDate", "Location", "fAllDayEvent", "fRecurrence","EventAttendees", "EventCommitteeAttendees",  "ParticipantsPicker","Administrative_x0020_Items", "TaxKeyword", "ApprovedApplicationLookup", "IACUCMeetingMinuteLookup", "RDCMeetingMinuteLookup", "Decision", 
        "Deliberation"
        )
	    $ct = $newList.ContentTypes[$contentType] 	    
	    $ct.FieldLinks.Reorder($newFieldOrder)
	    $ct.Update() 
	    $newList.Update()
        #set as default view
        $newView.DefaultView = $true
        $newView.Update()
        #migrate data from old list to new list
        $newItem = $null
        foreach($oldItem in $oldList.Items) {
            $newItem = $newList.AddItem()
            foreach($fld in $oldItem.Fields) {
              if(($newList.Fields.ContainsField($fld.Title) -eq $True)) {  
                if("ID","WF","Link","_","Agenda","Attachments","Facilities" -notcontains $fld.StaticName){
                    if($fld.ReadOnlyField -eq $False) {
                        $newItem[$fld.StaticName] = $oldItem[$fld.StaticName]
                        Write-Host Item: $newItem[$fld.StaticName] has been added to the $newList -ForegroundColor Cyan            
                    }              
                 } 
               } 
             }
            $newItem.Update()            
        }
        $newList.Update()
    }
}

#endregion

#Custom Function to delete a column with PowerShell
Function Remove-ColumnFromList ($WebURL, $ListName, $ColumnName)
{
 
    #Get Internal Name of the columns
    $web = Get-SPWeb $WebURL
 
    #Get the list
    $list = $web.Lists.TryGetList($ListName)
 
    if($List -ne $null)
    {
        #Get the column
        $list.Fields | ? {$_.Title -match $ColumnName} | % {
            #Reset column properties to allow delete
            #$_.Hidden = $False
            $_.ReadOnlyField = $False
            $_.AllowDeletion = $True
            $_.Update()
            #Delete the column from list
            $list.Fields.Delete($_)
            write-host ("Column " + $_.Title + " has been deleted!") -f Magenta
        }
    }
    else
    {
        write-host "Specified List is not found!" -ForegroundColor Yellow
    }
}

function Set-SiteColumnLookup ($colName,$colStaticName, $lookupListName,$lookupColumnName) {
    $web = Get-SiteWeb
    $lookupList = $web.Lists[$lookupListName]  
    $web.Fields.AddLookup($colStaticName, $lookupList.ID,$false);
    $field = $web.Fields[$colStaticName]
    $field.LookupField = $lookupColumnName
    $field.Title = $colName
    $field.Update()
}

function Set-FacilityListPermissionAndVersioning ($listTitle,[Microsoft.SharePoint.SPMember]$securityMember,[Microsoft.SharePoint.SPRights]$permissionMask, [switch] $enableVersioning) {
    (Get-SiteWeb).webs | % {
        $listPerm = $_.Lists[$listTitle]
        $listPerm.Permissions.Add($securityMember,$permissionMask)
        #versioning
        if($enableVersioning) {
            $listPerm.EnableVersioning = $True            
        }
        $listPerm.Update()
    }
}

function Import-RAMSUsers ($inputFilePath) {
    $weburl = Get-SiteURL
    $webSite = Get-SiteWeb

    $userList = import-csv -Path $inputFilePath -Delimiter "," | Select-Object -Property "Security Group", Person

    $userList | % {    
        $group = $_."Security Group"
        $person = $_.Person
        $userGroup = $webSite.SiteGroups | ? {$_.Name -eq ($group)}
        $matchUser = $webSite.SiteUsers | ? {$_.DisplayName -eq $person}
        if($matchUser) {
            $matchUser | % {
                $userGroup.AddUser($_)
                Write-Host "$($matchUser.DisplayName) has been added to Security Group: $($userGroup.Name)" -ForegroundColor Cyan
            }
        }
    }
}
#region Calendar related functions for RAMS
function New-MeetingMinuteLookup ([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$committee) {
    # GET THE SITE
    #$web = Get-SPWeb $siteURL    

    #CREATE SITE COLUMNS
    echo "CREATING MeetingMinuteLookup SITE COLUMNS"
    
    $calendarName=$committee + " Calendar"
	# GET THE LISTS
	$myList = $web.Lists[$calendarName];

	#Where the Source List for Lookup Exists
	
	$ParentWebURL=$web.Url
	
    $ParentListName=$committee + " Documents"
	$ParentLookupColumnName="Title"
	
	#Where the Lookup Site column Going to get created
	$ChildWebURL=$web.Url
	
    $ChildLookupColumnName= $committee + "MeetingMinuteLookup"

	#Get the Parent and Child Webs and ListQ
	
	$ParentWeb = Get-SPWeb $ParentWebURL
	$ParentList = $ParentWeb.Lists[$ParentListName]
	$ChildWeb = Get-SPWeb $ChildWebURL
	
	if(!$ChildWeb.Fields.ContainsField($ChildLookupColumnName))
	
	{
	    #Add Lookup Field
	
	    $ChildLookupColumn = $ChildWeb.Fields.AddLookup($ChildLookupColumnName,$ParentList.id,$False)
	
	    $ChildLookupColumn = $ChildWeb.Fields[$ChildLookupColumnName]
	
	    #Setup lookup Field property
	
	    $ChildLookupColumn.LookupWebId = $ParentWeb.ID
	
	    $ChildLookupColumn.LookupField = $ParentList.Fields[$ParentLookupColumnName].InternalName
	
	    $ChildLookupColumn.AllowMultipleValues=$true
	
	   	$ChildLookupColumn.Group="RAMS IRB" 
        $ChildLookupColumn.Title=$committee + " Meeting Minute Lookup"

	    $ChildLookupColumn.update()	

        write-host "Lookup field added successfully to the site!" -f green
	}
	else
	{
       write-host "Field Exists already on the web!" -f green
       $ChildLookupColumn = $ChildWeb.Fields.TryGetFieldByStaticName($ChildLookupColumnName)
       #Add column to the Calendar List, if missing
        if($myList.Fields.ContainsFieldWithStaticName($ChildLookupColumnName) -eq $False)
        {    
	        $myList.Fields.Add($ChildLookupColumn)
	        $myList.Update()
 	        write-host "Lookup field added successfully to the Calendar List!" -f green
        }
	
	} 

}

function Update-CalendarView([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$committee) 
{
 
    $list = $web.Lists.TryGetList("$committee Calendar")
    if($list -ne $null)
    {
      #UPDATE THE VIEW
      echo "UPDATING THE VIEW"
      $view = $list.Views | ? { $_.Title -eq 'All Events' };
 	
 	    if( $view.ViewFields.exists("Location")){
       		$view.ViewFields.Delete("Location");
       	}

     
       if( $view.ViewFields.exists("EventDate")){
      	$view.ViewFields.Delete("EventDate");
       }

	  if( $view.ViewFields.exists("EndDate")){
        $view.ViewFields.Delete("EndDate");
       }
       
       if( $view.ViewFields.exists("fAllDayEvent")){
        $view.ViewFields.Delete("fAllDayEvent");
       
       }

	   #Create customized view
	   if( !$view.ViewFields.exists("EventDate")){
        $view.ViewFields.Add("EventDate"); 
       }
       
  	   if( !$view.ViewFields.exists("EndDate")){
        $view.ViewFields.Add("EndDate"); 
       }
       
      if( !$view.ViewFields.exists("Location")){
        $view.ViewFields.Add("Location"); 
       }
	
	  if( !$view.ViewFields.exists("EventAttendees")){
        $view.ViewFields.Add("EventAttendees"); 
       }
       
       if( !$view.ViewFields.exists("Committee Attendees")){
        $view.ViewFields.Add("Committee Attendees"); 
       }
  
        $view.DefaultView = $true
        $view.Update()

        if($list -ne $null)
        {
     
            $contentType = "Event"
            $newFieldOrder = @("Title", "EventDate", "EndDate", "Location", "fAllDayEvent", "fRecurrence","EventAttendees", "EventCommitteeAttendees",  "ParticipantsPicker","Administrative_x0020_Items", "TaxKeyword", "SubmittedApplicationLookup", "$($committee)MeetingMinuteLookup", "Decision", 
            "Deliberation"
            )
           if ($list.Fields.ContainsField("Description")) {
	            $ct = $list.ContentTypes[$contentType] 
	            $ct.FieldLinks["Description"].Hidden = $True
	            $ct.FieldLinks.Reorder($newFieldOrder)
	            $ct.Update() 
	            $list.Update()
           }

          if ($list.Fields.ContainsField("Category")) {
	            $categoryfield=$list.Fields["Category"]
	            $categoryfield.ShowInNewForm=$false 
	            $categoryfield.ShowInEditForm=$false
	            $categoryfield.Update($true)
	            $list.Update()
       	 }
       
      	  if ($list.Fields.ContainsField("ApplicationsLookup")) {
      		$applicationsField=$list.Fields["ApplicationsLookup"]
            $applicationsField.ShowInNewForm=$false 
            $applicationsField.ShowInEditForm=$false
            $applicationsField.Update($true)
            $list.Update()
      	  }
      	  
      	  if ($list.Fields.ContainsField("RDCMinuteLookup")) {
      	    $approvedRDCApplicationLookup=$list.Fields["RDCMinuteLookup"]
            $approvedRDCApplicationLookup.ShowInNewForm=$false 
            $approvedRDCApplicationLookup.ShowInEditForm=$false
            $approvedRDCApplicationLookup.Update($true)
            $list.Update()
      	  }

          if ($list.Fields.ContainsField("Approved Application Lookup")) {
      		$applicationsField=$list.Fields["Approved Application Lookup"]
            $applicationsField.ShowInNewForm=$false 
            $applicationsField.ShowInEditForm=$false
            $applicationsField.Update($true)
            $list.Update()
      	  }

         if ($list.Fields.ContainsField("Attendees")) {
      		$applicationsField=$list.Fields["Attendees"]
            $applicationsField.ShowInNewForm=$false 
            $applicationsField.ShowInEditForm=$false
            $applicationsField.Update($true)
            $list.Update()
      	  }

         if ($list.Fields.ContainsField("Administrative Items")) {
      		$applicationsField=$list.Fields["Administrative Items"]
            $applicationsField.ShowInNewForm=$false 
            $applicationsField.ShowInEditForm=$false
            $applicationsField.Update($true)
            $list.Update()
      	  }
        }	

    }	
}  
#endregion

function New-SPList ([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$newCommiteeName,[Parameter (Mandatory=$True)][ValidateSet("Documents", "Calendar", "Form Library")][string]$templateName ) {
    
    if($templateName -eq "Documents") {
        $template = $website.ListTemplates["Document Library"]
    }
    else {
        $template = $website.ListTemplates["$templateName"]
    }
    
       

    #add new list
    if(($facilitySite.Lists.TryGetList("$($newCommiteeName)$templateName")) -eq $Null) {
        $facilitySite.Lists.Add("$($newCommiteeName)$templateName","$($newCommiteeName) $templateName",$template)    
    }
    $newSPList = $facilitySite.Lists.TryGetList("$($newCommiteeName)$templateName")
              
    #change title
    $newSPList.Title="$($newCommiteeName) $templateName";
        
    #enable versioning
    $newSPList.EnableVersioning = $True;        
        
    #update calendar
    $newSPList.Update() 

    #return
    return $newSPList
}

Export-ModuleMember -function Get-SiteURL 
Export-ModuleMember -function Get-SiteWeb 
Export-ModuleMember -function Set-SiteColumn -Alias Add-SiteColumn, asc
Export-ModuleMember -function Add-SiteColumnToCT -Alias ascct

Export-ModuleMember -function Add-FacilityWorkflow -Alias afwf
Export-ModuleMember -function Add-SiteCollectionWorkFlow
Export-ModuleMember -function Get-DateFromDays
Export-ModuleMember -function Format-ListOutput
Export-ModuleMember -function Get-ListInternalName
Export-ModuleMember -function Get-SPList
Export-ModuleMember -function Remove-DataFromAList
Export-ModuleMember -function Deploy-Feature
Export-ModuleMember -function Update-UtilMF
Export-ModuleMember -function Get-RAMSEnv
Export-ModuleMember -function New-RAMSEnv
Export-ModuleMember -function Update-RAMSEnv
Export-ModuleMember -function Get-RAMSSharePointInstallInfo
Export-ModuleMember -function Get-FacilitySites
Export-ModuleMember -function New-RAMSGroup -Alias nrg
Export-ModuleMember -function Add-FacilityGroup -Alias afg
Export-ModuleMember -Function RunSQL-ReturnInt -Alias rsri
Export-ModuleMember -Function RunSQL-ReturnDR -Alias rsrdr
Export-ModuleMember -Function Remove-WorkFlow -Alias remwf
Export-ModuleMember -Function Rename-List -Alias renList
Export-ModuleMember -Function Clone-Calendar -Alias cloneCal
Export-ModuleMember -Function Remove-CalendarColumns -Alias RCC

Export-ModuleMember -Function Remove-FacilitySitePages 
Export-ModuleMember -Function Remove-FacilitySiteAssets 

#added on 03/16/2018
Export-ModuleMember -Function New-SpListView
Export-ModuleMember -Function Update-SpListView

#added on 03/17/2018
Export-ModuleMember -function Remove-ContentTypeFromList 
#Export-ModuleMember -function Set-SiteColumnLookup
Export-ModuleMember -function Add-SiteColumnToCTByTitle

#added on 03/19/2018
Export-ModuleMember -function Remove-ColumnFromList

#added on 04/10/2018
Export-ModuleMember -function Set-SiteColumnLookup

#added on 04/13/2018
Export-ModuleMember -function Update-SiteColumnLookup

#added on 4/15/2018
Export-ModuleMember -Function Remove-SitePages 
Export-ModuleMember -Function Remove-SiteAssets 

#added on 5/01/2018
Export-ModuleMember -Function Set-FacilityListPermissionAndVersioning

#added on 5/07/2018
Export-ModuleMember -Function Import-RAMSUsers

#added on 5/10/2018
Export-ModuleMember -Function New-MeetingMinuteLookup
Export-ModuleMember -Function Update-CalendarView
Export-ModuleMember -Function New-SPList


<#
$PSModuleAutoLoadingPreference
Remove-Module -Name RAMS_Module_Util
Import-Module .\module\RAMS_Module_Util.psm1
New-RAMSEnv
Update-RAMSEnv

Set-Location .\UtilityScripts
Set-Location .\module
New-ModuleManifest -Path .\RAMS_Module_Util.psd1 -ModuleVersion 1.1.0 -Author "Chita Nilak, ByLight" -RootModule .\RAMS_Module_Util.psm1 -Description "RAMS Utility Functions" -CompanyName "VA.GOV.RAMS" -Verbose 

if((Get-Module -Name RAMS_Module_Util) -ne $null) {
    Remove-Module -Name RAMS_Module_Util
}
try {
    Import-Module .\RAMS_Module_Util.psd1 
}
catch {
    Write-Host Host: $env:COMPUTERNAME is not a Server, SharePoint is not installed on this computer...ignore, information only.
    #do nothing.
}
clear
#>


